@@ -6,7 +6,7 @@
  * Descr.: boolean and Meta-constraint predicate management                * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_bool.pl,v 1.5 2001/09/17 16:01:50 diaz Exp $ */
+/* $Id: fd_bool.pl,v 1.7 2002/03/19 18:24:33 diaz Exp $ */
 
 :-	built_in_fd.
 
@@ -78,7 +78,7 @@ LE #\\/ RE :-
 
 
 
-        /* Symbolic boolean constraints */
+        % Symbolic boolean constraints
 
 
 fd_cardinality(List, Count) :-
