@@ -6,7 +6,7 @@
  * Descr.: arithmetic (inline) management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arith_inl_c.c,v 1.5 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: arith_inl_c.c,v 1.7 2002/03/19 18:24:33 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -56,7 +56,7 @@
 typedef struct
 {
   WamWord f_n;
-  WamWord (*fct) ();
+  WamWord (*fct) () FC;
 }
 ArithInf;
 
@@ -88,13 +88,13 @@ static WamWord Load_Math_Expression(WamWord exp);
 
 #define C_Neg(x)     (- (x))
 
-#define C_Add(x,y)   ((x) + (y))
+#define C_Add(x, y)  ((x) + (y))
 
-#define C_Sub(x,y)   ((x) - (y))
+#define C_Sub(x, y)  ((x) - (y))
 
-#define C_Mul(x,y)   ((x) * (y))
+#define C_Mul(x, y)  ((x) * (y))
 
-#define C_Div(x,y)   ((y) != 0 ? (x) / (y) : (Pl_Err_Evaluation(evluation_zero_divisor), 0))
+#define C_Div(x, y)  ((y) != 0 ? (x) / (y) : (Pl_Err_Evaluation(evluation_zero_divisor), 0))
 
 #define Identity(x)  (x)
 
