@@ -6,7 +6,7 @@
  * Descr.: atom manipulation management - C part                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: atom_c.c,v 1.5 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: atom_c.c,v 1.9 2002/04/10 13:10:43 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -50,9 +50,9 @@
  * Function Prototypes             *
  *---------------------------------*/
 
-static
-  Bool Compute_Next_BLA(int mask, AtomInf *patom, AtomInf *psub_atom,
-			int b, int l, int a, int *b1, int *l1, int *a1);
+static Bool
+Compute_Next_BLA(int mask, AtomInf *patom, AtomInf *psub_atom,
+		 int b, int l, int a, int *b1, int *l1, int *a1);
 
 
 
@@ -74,10 +74,10 @@ Prolog_Prototype(ATOM_CONCAT_ALT, 0);
 Prolog_Prototype(SUB_ATOM_ALT, 0);
 Prolog_Prototype(CURRENT_ATOM_ALT, 0);
 
-#define MALLOC_STR(n) \
-   if (n<0) \
-     return FALSE; \
-   str=(char *) Malloc(n+1)
+#define MALLOC_STR(n)				\
+   if (n<0)					\
+     return FALSE;				\
+   str = (char *) Malloc(n + 1)
 
 
 
@@ -832,8 +832,8 @@ String_To_Number(char *str, WamWord number_word)
     }
 #endif
 
-  stm = Add_Str_Stream(TRUE, str);
-  pstm = stm_tbl + stm;
+  stm = Add_Str_Stream(str, TERM_STREAM_ATOM);
+  pstm = stm_tbl[stm];
 
   word = Read_Number(pstm);
   eof = (Stream_Peekc(pstm) == EOF);
