@@ -6,7 +6,7 @@
  * Descr.: character input-output management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: char_io_c.c,v 1.3 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: char_io_c.c,v 1.6 2002/04/09 08:26:39 diaz Exp $ */
 
 #include <stdio.h>
 
@@ -53,11 +53,11 @@
 
 
 /*-------------------------------------------------------------------------*
- * GET_CODE_NO_ECHO_2                                                      *
+ * GET_KEY_2                                                               *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Get_Code_No_Echo_2(WamWord sora_word, WamWord code_word)
+Get_Key_2(WamWord sora_word, WamWord code_word)
 {
   int stm;
   int c;
@@ -68,18 +68,9 @@ Get_Code_No_Echo_2(WamWord sora_word, WamWord code_word)
   last_input_sora = sora_word;
   Check_Stream_Type(stm, TRUE, TRUE);
 
-#if 0				/* not tested to allow for function keys */
-  Check_For_Un_Code(code_word);
-#else
   Check_For_Un_Integer(code_word);
-#endif
 
-
-  c = Stream_Getc_No_Echo(stm_tbl + stm);
-#if 0				/* not tested to allow for function keys */
-  if (c != EOF && !Is_Valid_Code(c))
-    Pl_Err_Representation(representation_character);
-#endif
+  c = Stream_Get_Key(stm_tbl[stm], TRUE, TRUE);
 
   if (c == EOF)
     c = -1;
@@ -91,13 +82,55 @@ Get_Code_No_Echo_2(WamWord sora_word, WamWord code_word)
 
 
 /*-------------------------------------------------------------------------*
- * GET_CODE_NO_ECHO_1                                                      *
+ * GET_KEY_1                                                               *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 Bool
-Get_Code_No_Echo_1(WamWord code_word)
+Get_Key_1(WamWord code_word)
 {
-  return Get_Code_No_Echo_2(NOT_A_WAM_WORD, code_word);
+  return Get_Key_2(NOT_A_WAM_WORD, code_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * GET_KEY_NO_ECHO_2                                                       *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Get_Key_No_Echo_2(WamWord sora_word, WamWord code_word)
+{
+  int stm;
+  int c;
+
+  stm = (sora_word == NOT_A_WAM_WORD)
+    ? stm_input : Get_Stream_Or_Alias(sora_word, STREAM_CHECK_INPUT);
+
+  last_input_sora = sora_word;
+  Check_Stream_Type(stm, TRUE, TRUE);
+
+  Check_For_Un_Integer(code_word);
+
+  c = Stream_Get_Key(stm_tbl[stm], FALSE, TRUE);
+
+  if (c == EOF)
+    c = -1;
+
+  return Get_Integer(c, code_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * GET_KEY_NO_ECHO_1                                                       *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Get_Key_No_Echo_1(WamWord code_word)
+{
+  return Get_Key_No_Echo_2(NOT_A_WAM_WORD, code_word);
 }
 
 
@@ -121,7 +154,7 @@ Get_Char_2(WamWord sora_word, WamWord char_word)
 
   Check_For_Un_In_Char(char_word);
 
-  c = Stream_Getc(stm_tbl + stm);
+  c = Stream_Getc(stm_tbl[stm]);
   if (c != EOF && !Is_Valid_Code(c))
     Pl_Err_Representation(representation_character);
 
@@ -162,7 +195,7 @@ Get_Code_2(WamWord sora_word, WamWord code_word)
 
   Check_For_Un_In_Code(code_word);
 
-  c = Stream_Getc(stm_tbl + stm);
+  c = Stream_Getc(stm_tbl[stm]);
   if (c != EOF && !Is_Valid_Code(c))
     Pl_Err_Representation(representation_character);
 
@@ -207,7 +240,7 @@ Get_Byte_2(WamWord sora_word, WamWord byte_word)
 
   Check_For_Un_In_Byte(byte_word);
 
-  c = Stream_Getc(stm_tbl + stm);
+  c = Stream_Getc(stm_tbl[stm]);
   if (c == EOF)
     c = -1;
 
@@ -247,7 +280,7 @@ Unget_Char_2(WamWord sora_word, WamWord char_word)
   last_input_sora = sora_word;
   Check_Stream_Type(stm, TRUE, TRUE);
 
-  Stream_Ungetc(Rd_Char_Check(char_word), stm_tbl + stm);
+  Stream_Ungetc(Rd_Char_Check(char_word), stm_tbl[stm]);
 }
 
 
@@ -281,7 +314,7 @@ Unget_Code_2(WamWord sora_word, WamWord code_word)
   last_input_sora = sora_word;
   Check_Stream_Type(stm, TRUE, TRUE);
 
-  Stream_Ungetc(Rd_Code_Check(code_word), stm_tbl + stm);
+  Stream_Ungetc(Rd_Code_Check(code_word), stm_tbl[stm]);
 }
 
 
@@ -316,7 +349,7 @@ Unget_Byte_2(WamWord sora_word, WamWord byte_word)
   last_input_sora = sora_word;
   Check_Stream_Type(stm, FALSE, TRUE);
 
-  Stream_Ungetc(Rd_Byte_Check(byte_word), stm_tbl + stm);
+  Stream_Ungetc(Rd_Byte_Check(byte_word), stm_tbl[stm]);
 }
 
 
@@ -353,7 +386,7 @@ Peek_Char_2(WamWord sora_word, WamWord char_word)
 
   Check_For_Un_In_Char(char_word);
 
-  c = Stream_Peekc(stm_tbl + stm);
+  c = Stream_Peekc(stm_tbl[stm]);
   if (c != EOF && !Is_Valid_Code(c))
     Pl_Err_Representation(representation_character);
 
@@ -394,7 +427,7 @@ Peek_Code_2(WamWord sora_word, WamWord code_word)
 
   Check_For_Un_In_Code(code_word);
 
-  c = Stream_Peekc(stm_tbl + stm);
+  c = Stream_Peekc(stm_tbl[stm]);
   if (c != EOF && !Is_Valid_Code(c))
     Pl_Err_Representation(representation_character);
 
@@ -438,7 +471,7 @@ Peek_Byte_2(WamWord sora_word, WamWord byte_word)
 
   Check_For_Un_In_Byte(byte_word);
 
-  c = Stream_Peekc(stm_tbl + stm);
+  c = Stream_Peekc(stm_tbl[stm]);
   if (c == EOF)
     c = -1;
 
@@ -476,7 +509,7 @@ Put_Char_2(WamWord sora_word, WamWord char_word)
   last_output_sora = sora_word;
   Check_Stream_Type(stm, TRUE, FALSE);
 
-  Stream_Putc(Rd_Char_Check(char_word), stm_tbl + stm);
+  Stream_Putc(Rd_Char_Check(char_word), stm_tbl[stm]);
 }
 
 
@@ -510,7 +543,7 @@ Put_Code_2(WamWord sora_word, WamWord code_word)
   last_output_sora = sora_word;
   Check_Stream_Type(stm, TRUE, FALSE);
 
-  Stream_Putc(Rd_Code_Check(code_word), stm_tbl + stm);
+  Stream_Putc(Rd_Code_Check(code_word), stm_tbl[stm]);
 }
 
 
@@ -545,7 +578,7 @@ Put_Byte_2(WamWord sora_word, WamWord byte_word)
   last_output_sora = sora_word;
   Check_Stream_Type(stm, FALSE, FALSE);
 
-  Stream_Putc(Rd_Byte_Check(byte_word), stm_tbl + stm);
+  Stream_Putc(Rd_Byte_Check(byte_word), stm_tbl[stm]);
 }
 
 
