@@ -6,7 +6,7 @@
  * Descr.: debugger                                                        * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: debugger.pl,v 1.6 2001/09/28 18:26:47 diaz Exp $ */
+/* $Id: debugger.pl,v 1.8 2002/03/19 18:24:35 diaz Exp $ */
 
 :-	built_in.
 
@@ -55,10 +55,7 @@
 
 
 
-          /*---------------------------------------------------------------* 
-           * Debugger built-in predicates                                  * 
-           *                                                               * 
-           *---------------------------------------------------------------*/
+           % Debugger built-in predicates
 
 wam_debug :-
 	set_bip_name(wam_debug, 0),
@@ -439,12 +436,10 @@ nospyall.
 
 
 
-          /*---------------------------------------------------------------* 
-           * The debugger:                                                 * 
-           *                                                               * 
-           * '$debug_call'/2 is called by meta-call (cf Call_2()) when the * 
-           * debugger is active, ie. Set_Debug_Call_Code() has been called * 
-           *---------------------------------------------------------------*/
+          % The debugger:
+          %
+          % '$debug_call'/2 is called by meta-call (cf Call_2()) when the
+          % debugger is active, ie. Set_Debug_Call_Code() has been called
 
 '$debug_call'(notrace, _) :-
 	!,
@@ -562,7 +557,7 @@ nospyall.
 
 
 
-          /* debug_port */
+          % debug_port
 
 '$debug_port'(Goal, Invoc, Index, AncLst, Port) :-
 	'$get_current_B'(B),
@@ -638,7 +633,7 @@ nospyall.
 '$debug_read_cmd'(C) :-
 	write(debugger_output, ' ? '),
 	flush_output(debugger_output),
-	get_code_no_echo(debugger_input, X),
+	get_key(debugger_input, X),
 	X >= 0,
 	X < 255,
 	char_code(C, X),
