@@ -6,7 +6,7 @@
  * Descr.: dynamic predicate support                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: dynam_supp.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: dynam_supp.c,v 1.7 2002/04/09 08:26:39 diaz Exp $ */
 
 #include <stdlib.h>
 
@@ -237,7 +237,7 @@ Add_Dynamic_Clause(WamWord head_word, WamWord body_word, Bool asserta,
 
   if ((pred = Lookup_Pred(func, arity)) == NULL)
     pred = Create_Pred(func, arity, atom_user_input,
-		       stm_tbl[stm_stdin].line_count,
+		       stm_tbl[stm_stdin]->line_count,
 		       MASK_PRED_DYNAMIC | MASK_PRED_PUBLIC, NULL);
   else if (check_perm && !(pred->prop & MASK_PRED_DYNAMIC))
     {
