@@ -6,7 +6,7 @@
  * Descr.: Prolog errors support - header file                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: error_supp.h,v 1.4 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: error_supp.h,v 1.7 2002/04/19 15:43:35 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -78,13 +78,13 @@ int domain_stream_property;
 int domain_write_option;
 int domain_term_stream_or_alias;	/* for term_streams */
 int domain_g_array_index;	/* for g_vars */
+int domain_g_argument_selector;	/* for g_vars */
 int domain_stream_seek_method;	/* for seek/4 */
 int domain_format_control_sequence;	/* for format/2-3 */
 int domain_os_path;		/* for absolute_file_name/2 */
 int domain_os_file_permission;	/* for file_permission/2 */
 int domain_selectable_item;	/* for select_read/3 */
 int domain_date_time;		/* for os_interf */
-
 #ifndef NO_USE_SOCKETS
 int domain_socket_domain;	/* for sockets */
 int domain_socket_address;	/* for sockets */
@@ -93,7 +93,7 @@ int domain_socket_address;	/* for sockets */
 int existence_procedure;
 int existence_source_sink;
 int existence_stream;
-
+int existence_sr_descriptor;	/* for source reader */
 
 int permission_operation_access;
 int permission_operation_close;
@@ -123,8 +123,7 @@ int representation_max_arity;
 int representation_max_integer;
 int representation_min_integer;
 int representation_too_many_variables;
-
-						    /* for Copy_Term(),... */
+				/* for Copy_Term(),... */
 
 int evluation_float_overflow;
 int evluation_int_overflow;
@@ -133,9 +132,8 @@ int evluation_underflow;
 int evluation_zero_divisor;
 
 
-int resource_too_many_open_streams;	/* for streams */
-int resource_print_object_not_linked;	/* for print and format */
-int resource_too_big_fd_constraint;	/* for FD */
+int resource_print_object_not_linked; /* for print and format */
+int resource_too_big_fd_constraint; /* for FD */
 
 
 #else
@@ -181,22 +179,23 @@ extern int domain_stream_property;
 extern int domain_write_option;
 extern int domain_term_stream_or_alias;	/* for term_streams */
 extern int domain_g_array_index;	/* for g_vars */
+extern int domain_g_argument_selector;	/* for g_vars */
 extern int domain_stream_seek_method;	/* for seek/4 */
 extern int domain_format_control_sequence;	/* for format/2-3 */
 extern int domain_os_path;	/* for absolute_file_name/2 */
-extern int domain_os_file_permission;	/* for file_permission/2 */
-extern int domain_selectable_item;	/* for select_read/3 */
+extern int domain_os_file_permission; /* for file_permission/2 */
+extern int domain_selectable_item; /* for select_read/3 */
 extern int domain_date_time;	/* for os_interf */
-
 #ifndef NO_USE_SOCKETS
-extern int domain_socket_domain;	/* for sockets */
-extern int domain_socket_address;	/* for sockets */
+extern int domain_socket_domain; /* for sockets */
+extern int domain_socket_address; /* for sockets */
 #endif
 
 
 extern int existence_procedure;
 extern int existence_source_sink;
 extern int existence_stream;
+extern int existence_sr_descriptor; /* for source reader */
 
 
 extern int permission_operation_access;
@@ -227,8 +226,7 @@ extern int representation_max_arity;
 extern int representation_max_integer;
 extern int representation_min_integer;
 extern int representation_too_many_variables;
-
-						    /* for Copy_Term(),... */
+				/* for Copy_Term(),... */
 
 
 extern int evluation_float_overflow;
@@ -238,9 +236,9 @@ extern int evluation_underflow;
 extern int evluation_zero_divisor;
 
 
-extern int resource_too_many_open_streams;	/* for streams */
-extern int resource_print_object_not_linked;	/* for print and format */
-extern int resource_too_big_fd_constraint;	/* for FD */
+extern int resource_too_many_open_streams; /* for streams */
+extern int resource_print_object_not_linked; /* for print and format */
+extern int resource_too_big_fd_constraint; /* for FD */
 
 #endif
 
