@@ -6,7 +6,7 @@
  * Descr.: Prolog flag and system variable management                      * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: flag.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: flag.pl,v 1.7 2002/04/09 08:26:39 diaz Exp $ */
 
 :-	built_in.
 
@@ -74,6 +74,9 @@ current_prolog_flag(Flag, Value) :-
 '$sys_var_reset_bit'(Var, Bit) :-
 	'$call_c'('Sys_Var_Reset_Bit_2'(Var, Bit)).
 
+'$sys_var_get_bit'(Var, Bit, Value) :-
+	'$call_c_test'('Sys_Var_Get_Bit_3'(Var, Bit, Value)).
+
 
 
 
