@@ -6,7 +6,7 @@
  * Descr.: formatted output management - C part                            *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: format_c.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: format_c.c,v 1.7 2002/04/09 08:26:40 diaz Exp $ */
 
 #include <string.h>
 #include <stdio.h>
@@ -79,7 +79,7 @@ Format_3(WamWord sora_word, WamWord format_word, WamWord args_word)
 
   stm = (sora_word == NOT_A_WAM_WORD)
     ? stm_output : Get_Stream_Or_Alias(sora_word, STREAM_CHECK_OUTPUT);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   last_output_sora = sora_word;
   Check_Stream_Type(stm, TRUE, FALSE);
@@ -94,7 +94,7 @@ Format_3(WamWord sora_word, WamWord format_word, WamWord args_word)
       str = buff;
     }
 
-  Format(stm_tbl + stm, str, &args_word);
+  Format(stm_tbl[stm], str, &args_word);
 }
 
 
