@@ -6,7 +6,7 @@
  * Descr.: global variable (inline) management - defs for meta-call        * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: g_var_inl.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: g_var_inl.pl,v 1.7 2002/04/19 15:43:35 diaz Exp $ */
 
 :-	built_in.
 
@@ -55,3 +55,64 @@ g_read(Var, Value) :-
 
 g_array_size(Var, Type) :-
 	g_array_size(Var, Type).
+
+
+
+g_inc(X) :-
+	g_inc(X).
+
+
+g_inco(X, Y) :-
+	g_inco(X, Y).
+
+
+g_inc(X, Y) :-
+	g_inc(X, Y).
+
+
+g_inc(X, Y, Z) :-
+	g_inc(X, Y, Z).
+
+
+
+
+g_dec(X) :-
+	g_dec(X).
+
+
+g_deco(X, Y) :-
+	g_deco(X, Y).
+
+
+g_dec(X, Y) :-
+	g_dec(X, Y).
+
+
+g_dec(X, Y, Z) :-
+	g_dec(X, Y, Z).
+
+
+
+
+g_set_bit(X, Y) :-
+	g_set_bit(X, Y).
+
+
+
+
+g_reset_bit(X, Y) :-
+	g_reset_bit(X, Y).
+
+
+
+
+g_test_set_bit(X, Y) :-
+	g_test_set_bit(X, Y).
+
+
+
+
+g_test_reset_bit(X, Y) :-
+	g_test_reset_bit(X, Y).
+
+
