@@ -6,7 +6,7 @@
  * Descr.: parser support                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: parse_supp.c,v 1.5 2001/01/23 12:57:51 diaz Exp $ */
+/* $Id: parse_supp.c,v 1.8 2002/04/19 15:43:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -104,15 +104,15 @@ static void Parse_Error(char *err_msg);
 
 
 
-#define   Unget_Token           tok_present=TRUE
+#define Unget_Token           tok_present = TRUE
 
 
 
-#define Update_Last_Read_Position                                           \
-    {                                                                       \
-     last_read_line=token.line;                                             \
-     last_read_col =token.col;                                              \
-    }
+#define Update_Last_Read_Position		\
+{						\
+  last_read_line = token.line;			\
+  last_read_col = token.col;			\
+}
 
 
 
@@ -131,8 +131,6 @@ Parse_Supp_Initializer(void)
   atom_back_quotes = Create_Atom("back_quotes");
   atom_full_stop = Create_Atom("full_stop");
   atom_extend = Create_Atom("extend");
-
-  parse_end_of_term = PARSE_END_OF_TERM_DOT;
 }
 
 
@@ -160,12 +158,17 @@ Read_Next_Token(Bool comma_is_punct)
  * READ_TERM                                                               *
  *                                                                         *
  * Returns a Prolog term as a WAM word or NOT_A_WAM_WORD on syntax error.  *
+ * parse_end_of_term controls the end of term (see parse_supp.h).          *
+ * Uses the value of: FLAG_DOUBLE_QUOTES, FLAG_BACK_QUOTES                 *
  *-------------------------------------------------------------------------*/
 WamWord
-Read_Term(StmInf *pstm)
+Read_Term(StmInf *pstm, int parse_end_of_term)
 {
   int jmp_val;
   WamWord term;
+  int save_use_le_prompt = use_le_prompt;
+
+  use_le_prompt = 1;
 
   parse_nb_var = 0;
   last_read_line = -1;
@@ -221,6 +224,7 @@ Read_Term(StmInf *pstm)
     }
 
 finish:
+  use_le_prompt = save_use_le_prompt;
   return term;
 }
 
@@ -286,8 +290,13 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
       break;
 
     case TOKEN_STRING:
-      flag_value = Flag_Value(FLAG_DOUBLE_QUOTES);
-      if (flag_value == FLAG_DOUBLE_QUOTES_ATOM)
+    case TOKEN_BACK_QUOTED:	/* undefined in ISO */
+      flag_value = (token.type == TOKEN_STRING) ? 
+	Flag_Value(FLAG_DOUBLE_QUOTES) :
+	Flag_Value(FLAG_BACK_QUOTES);
+
+      flag_value &= FLAG_AS_PART_MASK;
+      if (flag_value == FLAG_AS_ATOM)
 	{
 	  atom = Create_Allocate_Atom(token.name);
 	  goto a_name;
@@ -298,7 +307,7 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
       while (i--)
 	{
 	  term1 = Put_List();
-	  if (flag_value == FLAG_DOUBLE_QUOTES_CODES)
+	  if (flag_value == FLAG_AS_CODES)
 	    Unify_Integer(token.name[i]);
 	  else
 	    Unify_Atom(ATOM_CHAR(token.name[i]));
@@ -326,7 +335,6 @@ Parse_Term(int cur_prec, int context, Bool comma_is_punct)
       break;
 
     case TOKEN_NAME:
-    case TOKEN_BACK_QUOTED:	/* undefined in ISO */
       atom = Create_Allocate_Atom(token.name);
 
     a_name:
