@@ -6,7 +6,7 @@
  * Descr.: predicate manipulation management - C part                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred_c.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: pred_c.c,v 1.7 2002/04/09 08:26:40 diaz Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -83,7 +83,11 @@ Current_Predicate_2(WamWord pred_indic_word, WamWord hide_word)
 	return FALSE;
 
       if (arity >= 0)
-	return Lookup_Pred(func, arity) != NULL;
+	{
+	  pred = Lookup_Pred(func, arity);
+	  return pred && (!Flag_Value(FLAG_STRICT_ISO) ||
+			  !(pred->prop & MASK_PRED_ANY_BUILTIN));
+	}
     }
 
   all = (arity == -1);
