@@ -6,7 +6,7 @@
  * Descr.: pretty print clause management - C part                         *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pretty_c.c,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: pretty_c.c,v 1.8 2002/04/09 08:26:40 diaz Exp $ */
 
 #define OBJ_INIT Pretty_Initializer
 
@@ -140,7 +140,7 @@ Portray_Clause_2(WamWord sora_word, WamWord term_word)
 
   stm = (sora_word == NOT_A_WAM_WORD)
     ? stm_output : Get_Stream_Or_Alias(sora_word, STREAM_CHECK_OUTPUT);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   last_output_sora = sora_word;
   Check_Stream_Type(stm, TRUE, FALSE);
@@ -177,12 +177,14 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
 {
   WamWord word, tag_mask;
   WamWord arg_word[2];
+  int atom;
 
   if (Check_Structure(term_word, atom_clause, 2, arg_word))
     {
       Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, arg_word[0]);
       DEREF(arg_word[1], word, tag_mask);
-      if (tag_mask != TAG_ATM_MASK || UnTag_ATM(word) != atom_true)
+      atom = UnTag_ATM(word);
+      if (tag_mask != TAG_ATM_MASK || atom != atom_true)
 	{
 	  Stream_Puts(" :-", pstm);
 	  Start_Line(pstm, 0, ' ');
@@ -196,7 +198,8 @@ Portray_Clause(StmInf *pstm, WamWord term_word)
     {
       Write_Term(pstm, -1, 1200 - 1, WRITE_MASK, arg_word[0]);
       DEREF(arg_word[1], word, tag_mask);
-      if (tag_mask != TAG_ATM_MASK || UnTag_ATM(word) != atom_true)
+      atom = UnTag_ATM(word);
+      if (tag_mask != TAG_ATM_MASK || atom != atom_true)
 	{
 	  Stream_Puts(" -->", pstm);
 	  Start_Line(pstm, 0, ' ');
