@@ -6,7 +6,7 @@
  * Descr.: term print management                                           * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: print.pl,v 1.6 2001/09/28 18:26:47 diaz Exp $ */
+/* $Id: print.pl,v 1.8 2002/04/09 08:26:40 diaz Exp $ */
 
 :-	built_in.
 
@@ -55,5 +55,5 @@ print(SorA, Term) :-
 
 get_print_stream(Stream) :-
 	set_bip_name(get_print_stream, 1),
-	'$get_open_stm'(Stream, Stm),
-	'$sys_var_read'(6, Stm).
+	'$check_stream_or_var'(Stream, Stm),
+	'$call_c'('Get_Print_Stm_1'(Stm)).
