@@ -6,7 +6,7 @@
  * Descr.: term input (read/1 and friends) management                      * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: read.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: read.pl,v 1.9 2002/04/19 15:43:35 diaz Exp $ */
 
 :-	built_in.
 
@@ -47,11 +47,11 @@ read(SorA, Term) :-
 
           % option mask in sys_var[0]:
           %
-          %    b3          b2         b1         b0
-          %    0/1         0/1        0/1        0/1
-          % end_of_term singletons   var_names   vars
-          %   0=dot       0=false     0=false    0=false
-          %   1=eof       1=true      1=true     1=true
+          %    b3            b2          b1         b0
+          %    0/1           0/1         0/1        0/1
+          % end_of_term   singletons   var_names   vars
+          %   0=dot         0=false     0=false    0=false
+          %   1=eof         1=true      1=true     1=true
           %
           % syntax error action in sys_var[1]:
           %
@@ -70,6 +70,9 @@ read_term(Term, Options) :-
 
 read_term(SorA, Term, Options) :-
 	set_bip_name(read_term, 3),
+	'$read_term'(SorA, Term, Options).
+	
+'$read_term'(SorA, Term, Options) :-
 	'$set_read_defaults',
 	'$get_read_options'(Options, Vars, VarNames, SinglNames),
 	'$call_c_test'('Read_Term_5'(SorA, Term, Vars, VarNames, SinglNames)).
@@ -86,9 +89,9 @@ read_term(SorA, Term, Options) :-
 
 '$get_read_options'(Options, Vars, VarNames, SinglNames) :-
 	'$check_list'(Options),
-	g_link('$read_variables', []),
-	g_link('$read_variable_names', []),
-	g_link('$read_singletons', []),
+	g_assign('$read_variables', []),
+	g_assign('$read_variable_names', []),
+	g_assign('$read_singletons', []),
 	'$get_read_options1'(Options),
 	g_read('$read_variables', Vars),
 	g_read('$read_variable_names', VarNames),
