@@ -6,7 +6,7 @@
  * Descr.: sockets management - C part                                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,14 +22,20 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: sockets_c.c,v 1.4 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: sockets_c.c,v 1.10 2002/05/07 17:45:46 diaz Exp $ */
+#include "gp_config.h"
 
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
 #include <errno.h>
 #include <sys/types.h>
+#ifndef M_ix86_win32
+#include <unistd.h>
 #include <sys/socket.h>
+#else
+#include <io.h>
+#include <winsock2.h>
+#endif
 
        /* old versions of CYGWIN do not support AF_UNIX - modify next line */
 #if defined(__unix__) || defined(__CYGWIN__)
@@ -39,9 +45,11 @@
 #ifdef SUPPORT_AF_UNIX
 #include <sys/un.h>
 #endif
+#ifndef M_ix86_win32
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#endif
 
 #define OBJ_INIT Socket_Initializer
 
@@ -89,10 +97,27 @@ static
 static void
 Socket_Initializer(void)
 {
+#ifdef M_ix86_win32
+  WORD versReqstd = MAKEWORD( 2, 2);		// Current Winsock 2 DLL's
+  WSADATA wsaData;
+  int err;
+#endif
+
 #ifdef SUPPORT_AF_UNIX
   atom_AF_UNIX = Create_Atom("AF_UNIX");
 #endif
   atom_AF_INET = Create_Atom("AF_INET");
+
+#ifdef M_ix86_win32
+  if ((err = WSAStartup(versReqstd, &wsaData)) != 0 ||
+      wsaData.wVersion != versReqstd)
+    {
+      Stream_Printf(stm_tbl[stm_top_level_output],
+		    "warning: cannot find a usable WinSock DLL\n");
+      if (err == 0)
+	WSACleanup();
+    }
+#endif
 }
 
 
@@ -106,7 +131,11 @@ Bool
 Socket_2(WamWord domain_word, WamWord socket_word)
 {
   int domain;
+#ifdef M_ix86_win32
+  SOCKET sock;
+#else
   int sock;
+#endif
 
   domain = Rd_Atom_Check(domain_word);
   if (
@@ -126,7 +155,11 @@ Socket_2(WamWord domain_word, WamWord socket_word)
 #endif
     sock = socket(AF_INET, SOCK_STREAM, 0);
 
+#ifdef M_ix86_win32
+  Os_Test_Error(sock == INVALID_SOCKET);
+#else
   Os_Test_Error(sock == -1);
+#endif
 
   return Get_Integer(sock, socket_word);
 }
@@ -141,7 +174,11 @@ Socket_2(WamWord domain_word, WamWord socket_word)
 Bool
 Socket_Close_1(WamWord socket_word)
 {
+#ifndef M_ix86_win32
   int sock;
+#else
+  SOCKET sock;
+#endif
 
   sock = Rd_Integer_Check(socket_word);
   if (sock < 2)
@@ -150,7 +187,11 @@ Socket_Close_1(WamWord socket_word)
       Os_Test_Error(1);
     }
   else
+#ifndef M_ix86_win32
     Os_Test_Error(close(sock));
+#else
+    Os_Test_Error(closesocket(sock));
+#endif
 
   return TRUE;
 }
@@ -239,7 +280,7 @@ Socket_Bind_2(WamWord socket_word, WamWord address_word)
   if (tag_mask != TAG_REF_MASK)
     port = Rd_Integer_Check(word);
 
-  adr_in.sin_port = htons(port);
+  adr_in.sin_port = htons((unsigned short) port);
   adr_in.sin_family = AF_INET;
   adr_in.sin_addr.s_addr = INADDR_ANY;
 
@@ -332,7 +373,7 @@ Socket_Connect_4(WamWord socket_word, WamWord address_word,
     return FALSE;
 
   adr_in.sin_family = AF_INET;
-  adr_in.sin_port = htons(port);
+  adr_in.sin_port = htons((unsigned short) port);
   memcpy(&adr_in.sin_addr, host_entry->h_addr_list[0],
 	 host_entry->h_length);
 
@@ -452,37 +493,36 @@ static Bool
 Create_Socket_Streams(int sock, char *stream_name,
 		      int *stm_in, int *stm_out)
 {
-  StmProp prop;
   int fd;
   FILE *f_in, *f_out;
-  int file_name;
+  int atom;
+  int stm;
 
+#ifdef M_ix86_win32
+  int r;
+
+  Os_Test_Error((fd = _open_osfhandle(sock, _IOWRT)) == -1);
+  Os_Test_Error((r = _open_osfhandle(sock, _IOREAD)) == -1);
+  Os_Test_Error((f_in = fdopen(r, "r")) == NULL);
+  Os_Test_Error((f_out = fdopen(fd, "w")) == NULL);
+#else
   Os_Test_Error((fd = dup(sock)) < 0);
   Os_Test_Error((f_in = fdopen(sock, "rt")) == NULL);
   Os_Test_Error((f_out = fdopen(fd, "wt")) == NULL);
+#endif
 
-  file_name = Create_Allocate_Atom(stream_name);
+  atom = Create_Allocate_Atom(stream_name);
 
-  prop.mode = STREAM_MODE_READ;
-  prop.input = TRUE;
-  prop.output = FALSE;
-  prop.text = TRUE;
-  prop.reposition = FALSE;
-  prop.eof_action = STREAM_EOF_ACTION_RESET;
-  prop.buffering = STREAM_BUFFERING_BLOCK;
-  prop.tty = FALSE;
-  prop.special_close = FALSE;
-  prop.other = 4;
+  stm = Add_Stream_For_Stdio_Desc(f_in, atom, STREAM_MODE_READ, TRUE);
+  stm_tbl[stm]->prop.eof_action = STREAM_EOF_ACTION_RESET;
+  stm_tbl[stm]->prop.other = 4;
 
-  *stm_in = Add_Stream(file_name, (long) f_in, prop,
-		       NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+  *stm_in = stm;
 
-  prop.mode = STREAM_MODE_WRITE;
-  prop.input = FALSE;
-  prop.output = TRUE;
+  stm = Add_Stream_For_Stdio_Desc(f_out, atom, STREAM_MODE_WRITE, TRUE);
+  stm_tbl[stm]->prop.other = 4;
 
-  *stm_out = Add_Stream(file_name, (long) f_out, prop,
-			NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+  *stm_out = stm;
 
   return TRUE;
 }
