@@ -6,7 +6,7 @@
  * Descr.: stream selection and control management - C part                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_c.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: stream_c.c,v 1.11 2002/06/12 11:53:14 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,6 +33,10 @@
 #include "engine_pl.h"
 #include "bips_pl.h"
 
+#ifndef NO_USE_LINEDIT
+#include "linedit.h"
+#endif
+
 #if defined(M_ix86_cygwin) || defined(M_ix86_win32)
 #include <io.h>
 #endif
@@ -51,10 +55,6 @@
 
 #define TERM_STREAM_WRITE_BLOCK    1024
 
-#define TERM_STREAM_ATOM           0
-#define TERM_STREAM_CHARS          1
-#define TERM_STREAM_CODES          2
-
 
 
 
@@ -91,11 +91,12 @@ TermSInf;
  *---------------------------------*/
 
 #define CURRENT_STREAM_ALT         X2463757272656E745F73747265616D5F616C74
-
 #define CURRENT_ALIAS_ALT          X2463757272656E745F616C6961735F616C74
+#define CURRENT_MIRROR_ALT         X2463757272656E745F6D6972726F725F616C74
 
 Prolog_Prototype(CURRENT_STREAM_ALT, 0);
 Prolog_Prototype(CURRENT_ALIAS_ALT, 0);
+Prolog_Prototype(CURRENT_MIRROR_ALT, 0);
 
 
 
@@ -198,14 +199,16 @@ void
 Open_3(WamWord source_sink_word, WamWord mode_word, WamWord stm_word)
 {
   WamWord word, tag_mask;
+  int atom;
+  int mode;
+  Bool text;
   StmProp prop;
   char *path;
   int atom_file_name;
   int stm;
-  char open_str[10];
   FILE *f;
-  long offset;
   int mask = SYS_VAR_OPTION_MASK;
+  Bool reposition;
 
 
   DEREF(source_sink_word, word, tag_mask);
@@ -219,14 +222,21 @@ Open_3(WamWord source_sink_word, WamWord mode_word, WamWord stm_word)
   if ((path = M_Absolute_Path_Name(path)) == NULL)
     Pl_Err_Existence(existence_source_sink, source_sink_word);
 
-  prop = Get_Stream_Mode(mode_word, FALSE, open_str);
-
-  prop.text = mask & 1;
+  text = mask & 1;
   mask >>= 1;
+  
+  atom = Rd_Atom_Check(mode_word);
+  if (atom == atom_read)
+    mode = STREAM_MODE_READ;
+  else if (atom == atom_write)
+    mode = STREAM_MODE_WRITE;
+  else if (atom == atom_append)
+    mode = STREAM_MODE_APPEND;
+  else
+    Pl_Err_Domain(domain_io_mode, mode_word);
 
-  strcat(open_str, (prop.text) ? "t" : "b");
-
-  if ((f = fopen(path, open_str)) == NULL)
+  stm = Add_Stream_For_Stdio_File(path, mode, text);
+  if (stm < 0)
     {
       if (errno == ENOENT || errno == ENOTDIR)
 	Pl_Err_Existence(existence_source_sink, source_sink_word);
@@ -235,48 +245,42 @@ Open_3(WamWord source_sink_word, WamWord mode_word, WamWord stm_word)
 			  permission_type_source_sink, source_sink_word);
     }
 
-  prop.tty = (isatty(fileno(f)) != 0);
+  prop = stm_tbl[stm]->prop;
+  f = (FILE *) stm_tbl[stm]->file;
 
-  if ((mask & 2) == 0)		/* not specified - set to default */
-    prop.reposition = !(prop.tty);
-  else
-    prop.reposition = mask & 1;
-  mask >>= 2;
+				/* change properties wrt to specified ones */
 
-
-  if ((mask & 4) == 0)		/* not specified - set to default */
-    prop.eof_action = (prop.tty) ? STREAM_EOF_ACTION_RESET
-      : STREAM_EOF_ACTION_EOF_CODE;
-  else
-    prop.eof_action = mask & 3;
-  mask >>= 2;
-
-
-  if ((mask & 4) == 0)		/* not specified - set to default */
-    prop.buffering = (prop.tty) ? STREAM_BUFFERING_LINE
-      : STREAM_BUFFERING_BLOCK;
-  else
-    prop.buffering = mask & 3;
-  mask >>= 2;
-
-  if (prop.reposition)
+  if ((mask & 2) != 0)		/* reposition specified */
     {
-      offset = ftell(f);
-      if (fseek(f, 0, SEEK_END) != 0 && errno != 0)
+      reposition = mask & 1;
+      if (reposition && !prop.reposition)
 	{
+	  fclose(f);
 	  word = Put_Structure(atom_reposition, 1);
 	  Unify_Atom(atom_true);
 	  Pl_Err_Permission(permission_operation_open,
 			    permission_type_source_sink, word);
 	}
-      fseek(f, offset, SEEK_SET);
+
+      prop.reposition = reposition;
     }
+  mask >>= 2;
 
-  prop.special_close = FALSE;
-  prop.other = 0;
+  if ((mask & 4) != 0)		/* eof_action specified */
+      prop.eof_action = mask & 3;
+  mask >>= 3;
 
-  stm = Add_Stream(atom_file_name, (long) f, prop,
-		   NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+
+  if ((mask & 4) != 0)		/* buffering specified */
+    if (prop.buffering != (mask & 3))
+      {
+	prop.buffering = mask & 3;
+	Stdio_Set_Buffering(f, prop.buffering);
+      }
+  mask >>= 3;
+
+  stm_tbl[stm]->atom_file_name = atom_file_name;
+  stm_tbl[stm]->prop = prop;
 
   Get_Integer(stm, stm_word);
 }
@@ -331,6 +335,57 @@ Add_Stream_Alias_2(WamWord sora_word, WamWord alias_word)
 
 
 
+/*-------------------------------------------------------------------------*
+ * CHECK_VALID_MIRROR_1                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Check_Valid_Mirror_1(WamWord mirror_word)
+{
+  Get_Stream_Or_Alias(mirror_word, STREAM_CHECK_OUTPUT);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * ADD_STREAM_MIRROR_2                                                     *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Add_Stream_Mirror_2(WamWord sora_word, WamWord mirror_word)
+{
+  int stm;
+  int m_stm;
+
+  stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
+  m_stm = Get_Stream_Or_Alias(mirror_word, STREAM_CHECK_OUTPUT);
+
+  Add_Mirror_To_Stream(stm, m_stm);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * REMOVE_STREAM_MIRROR_2                                                  *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Remove_Stream_Mirror_2(WamWord sora_word, WamWord mirror_word)
+{
+  int stm;
+  int m_stm;
+
+  stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
+  m_stm = Get_Stream_Or_Alias(mirror_word, STREAM_CHECK_EXIST);
+
+  return Del_Mirror_From_Stream(stm, m_stm);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * SET_STREAM_TYPE_2                                                       *
  *                                                                         *
@@ -343,7 +398,7 @@ Set_Stream_Type_2(WamWord sora_word, WamWord is_text_word)
   int text;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   text = Rd_Integer_Check(is_text_word);
   if ((unsigned) text == pstm->prop.text)
@@ -358,7 +413,7 @@ Set_Stream_Type_2(WamWord sora_word, WamWord is_text_word)
   {
     FILE *f;
 
-    f = File_Star_Of_Stream(stm);
+    f = Stdio_Desc_Of_Stream(stm);
     if (f == NULL)
       return;
 
@@ -381,7 +436,7 @@ Set_Stream_Eof_Action_2(WamWord sora_word, WamWord action_word)
   StmInf *pstm;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (pstm->prop.output)
     Pl_Err_Permission(permission_operation_modify,
@@ -404,7 +459,7 @@ Set_Stream_Buffering_2(WamWord sora_word, WamWord buff_mode_word)
   StmInf *pstm;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   pstm->prop.buffering = Rd_Integer_Check(buff_mode_word);
   Set_Stream_Buffering(stm);
@@ -420,18 +475,21 @@ Set_Stream_Buffering_2(WamWord sora_word, WamWord buff_mode_word)
 void
 Close_1(WamWord sora_word)
 {
-  int stm;
-  StmInf *pstm;
-  StmProp prop;
-  int mask = SYS_VAR_OPTION_MASK;
+  int stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
+  Close_Stm(stm, SYS_VAR_OPTION_MASK & 1);
+}
+
+
+/*-------------------------------------------------------------------------*
+ * CLOSE_STM                                                               *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Close_Stm(int stm, Bool force)
+{
+  StmInf *pstm = stm_tbl[stm];
   int fd = 0;
 
-
-  stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-
-  pstm = stm_tbl + stm;
-  prop = pstm->prop;
-
   Stream_Flush(pstm);
 
   if (stm == stm_stdin || stm == stm_stdout)
@@ -456,7 +514,7 @@ Close_1(WamWord sora_word)
 
   if (Stream_Close(pstm) != 0)
     {
-      if ((mask & 1) == 0)
+      if (force == 0)
 	Pl_Err_System(Create_Atom(ERR_CANNOT_CLOSE_STREAM));
 
       /* else force close */
@@ -485,7 +543,7 @@ Flush_Output_1(WamWord sora_word)
 
   last_output_sora = sora_word;
 
-  Stream_Flush(stm_tbl + stm);
+  Stream_Flush(stm_tbl[stm]);
 }
 
 
@@ -512,33 +570,34 @@ Bool
 Current_Stream_1(WamWord stm_word)
 {
   WamWord word, tag_mask;
-  int i = 0;
+  int stm = 0;
 
 
   DEREF(stm_word, word, tag_mask);	/* either an INT or a REF */
   if (tag_mask == TAG_INT_MASK)
-    return stm_tbl[UnTag_INT(word)].file != 0;
+    {
+      stm = UnTag_INT(word);
+      return (stm >= 0 && stm <= stm_last_used && stm_tbl[stm] != NULL);
+    }
 
-
-  for (; i <= stm_last_used; i++)
-    if (stm_tbl[i].file != 0)
+  for (; stm <= stm_last_used; stm++)
+    if (stm_tbl[stm])
       break;
 
-  if (i >= stm_last_used)
+  if (stm >= stm_last_used)
     {
-      if (i > stm_last_used)
+      if (stm > stm_last_used)
 	return FALSE;
     }
   else				/* non deterministic case */
     {
       A(0) = stm_word;
-      A(1) = i + 1;
+      A(1) = stm + 1;
       Create_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_STREAM_ALT, 0),
 			  2);
     }
 
-  Get_Integer(i, stm_word);
-  return TRUE;
+  return Get_Integer(stm, stm_word);
 }
 
 
@@ -552,23 +611,21 @@ Bool
 Current_Stream_Alt_0(void)
 {
   WamWord stm_word;
-  int i;
+  int stm;
 
   Update_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_STREAM_ALT, 0), 0);
 
   stm_word = AB(B, 0);
-  i = AB(B, 1);
+  stm = AB(B, 1);
 
-
-
-  for (; i <= stm_last_used; i++)
-    if (stm_tbl[i].file != 0)
+  for (; stm <= stm_last_used; stm++)
+    if (stm_tbl[stm])
       break;
 
-  if (i >= stm_last_used)
+  if (stm >= stm_last_used)
     {
       Delete_Last_Choice_Point();
-      if (i > stm_last_used)
+      if (stm > stm_last_used)
 	return FALSE;
     }
   else				/* non deterministic case */
@@ -576,11 +633,10 @@ Current_Stream_Alt_0(void)
 #if 0 /* the following data is unchanged */
       AB(B, 0) = stm_word;
 #endif
-      AB(B, 1) = i + 1;
+      AB(B, 1) = stm + 1;
     }
 
-  Get_Integer(i, stm_word);
-  return TRUE;
+  return Get_Integer(stm, stm_word);
 }
 
 
@@ -597,7 +653,7 @@ Stream_Prop_File_Name_2(WamWord file_name_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  return Un_Atom_Check(stm_tbl[stm].atom_file_name, file_name_word);
+  return Un_Atom_Check(stm_tbl[stm]->atom_file_name, file_name_word);
 }
 
 
@@ -615,7 +671,7 @@ Stream_Prop_Mode_2(WamWord mode_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  switch (stm_tbl[stm].prop.mode)
+  switch (stm_tbl[stm]->prop.mode)
     {
     case STREAM_MODE_READ:
       atom = atom_read;
@@ -647,7 +703,7 @@ Stream_Prop_Input_1(WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  return stm_tbl[stm].prop.input;
+  return stm_tbl[stm]->prop.input;
 }
 
 
@@ -664,7 +720,7 @@ Stream_Prop_Output_1(WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  return stm_tbl[stm].prop.output;
+  return stm_tbl[stm]->prop.output;
 }
 
 
@@ -682,7 +738,7 @@ Stream_Prop_Type_2(WamWord type_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  atom = (stm_tbl[stm].prop.text) ? atom_text : atom_binary;
+  atom = (stm_tbl[stm]->prop.text) ? atom_text : atom_binary;
 
   return Un_Atom_Check(atom, type_word);
 }
@@ -702,7 +758,7 @@ Stream_Prop_Reposition_2(WamWord reposition_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  atom = (stm_tbl[stm].prop.reposition) ? atom_true : atom_false;
+  atom = (stm_tbl[stm]->prop.reposition) ? atom_true : atom_false;
 
   return Un_Atom_Check(atom, reposition_word);
 }
@@ -722,7 +778,7 @@ Stream_Prop_Eof_Action_2(WamWord eof_action_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  switch (stm_tbl[stm].prop.eof_action)
+  switch (stm_tbl[stm]->prop.eof_action)
     {
     case STREAM_EOF_ACTION_ERROR:
       atom = atom_error;
@@ -755,7 +811,17 @@ Stream_Prop_Buffering_2(WamWord buffering_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  switch (stm_tbl[stm].prop.buffering)
+#ifndef NO_USE_LINEDIT		/* if use_gui == 1 */
+  if (stm_tbl[stm]->file == (long) stdout && le_hook_get_line_buffering)
+    {
+      if ((*le_hook_get_line_buffering)())
+	stm_tbl[stm]->prop.buffering = STREAM_BUFFERING_LINE;
+      else
+	stm_tbl[stm]->prop.buffering = STREAM_BUFFERING_NONE;
+    }
+#endif
+
+  switch (stm_tbl[stm]->prop.buffering)
     {
     case STREAM_BUFFERING_NONE:
       atom = atom_none;
@@ -788,7 +854,7 @@ Stream_Prop_End_Of_Stream_2(WamWord end_of_stream_word, WamWord stm_word)
 
   stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
 
-  switch (Stream_End_Of_Stream(stm_tbl + stm))
+  switch (Stream_End_Of_Stream(stm_tbl[stm]))
     {
     case STREAM_EOF_NOT:
       atom = atom_not;
@@ -821,7 +887,7 @@ At_End_Of_Stream_1(WamWord sora_word)
   stm = (sora_word == NOT_A_WAM_WORD)
     ? stm_input : Get_Stream_Or_Alias(sora_word, STREAM_CHECK_INPUT);
 
-  return Stream_End_Of_Stream(stm_tbl + stm) != STREAM_EOF_NOT;
+  return Stream_End_Of_Stream(stm_tbl[stm]) != STREAM_EOF_NOT;
 }
 
 
@@ -951,6 +1017,72 @@ Current_Alias_Alt_0(void)
 
 
 
+/*-------------------------------------------------------------------------*
+ * CURRENT_MIRROR_2                                                        *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Current_Mirror_2(WamWord stm_word, WamWord m_stm_word)
+{
+  int stm = Rd_Integer_Check(stm_word);	/* stm is a valid stream entry */
+  StmInf *pstm = stm_tbl[stm];
+  StmLst *m = pstm->mirror;
+
+				/* From here, the code also works with     */
+				/* m = m_pstm->mirror_of. Could be used    */
+				/* if m_stm_word is given and not stm_word */
+  if (m == NULL)
+    return FALSE;
+
+  if (m->next != NULL) /* non deterministic case */
+    {
+      A(0) = stm;
+      A(1) = m_stm_word;
+      A(2) = (WamWord) m->next;
+      Create_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_MIRROR_ALT, 0),
+			  3);
+    }
+
+  return Get_Integer(m->stm, m_stm_word);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * CURRENT_MIRROR_ALT_0                                                    *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Current_Mirror_Alt_0(void)
+{
+  int stm;
+  WamWord m_stm_word;
+  StmLst *m;
+
+  Update_Choice_Point((CodePtr) Prolog_Predicate(CURRENT_MIRROR_ALT, 0), 0);
+
+  stm = AB(B, 0);
+  m_stm_word = AB(B, 1);
+  m = (StmLst *) AB(B, 2);
+
+  if (m->next)			/* non deterministic case */
+    {
+#if 0				/* the following data is unchanged */
+      AB(B, 0) = stm;
+      AB(B, 1) = m_stm_word;
+#endif
+      AB(B, 2) = (WamWord) m->next;
+    }
+  else
+    Delete_Last_Choice_Point();
+
+  return Get_Integer(m->stm, m_stm_word);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * STREAM_POSITION_2                                                       *
  *                                                                         *
@@ -967,7 +1099,7 @@ Stream_Position_2(WamWord sora_word, WamWord position_word)
 
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   Stream_Get_Position(pstm, p, p + 1, p + 2, p + 3);
 
@@ -1010,7 +1142,7 @@ Set_Stream_Position_2(WamWord sora_word, WamWord position_word)
 
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (!pstm->prop.reposition)
     Pl_Err_Permission(permission_operation_reposition,
@@ -1061,7 +1193,7 @@ Seek_4(WamWord sora_word, WamWord whence_word, WamWord offset_word,
 
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (!pstm->prop.reposition)
     Pl_Err_Permission(permission_operation_reposition,
@@ -1108,7 +1240,7 @@ Character_Count_2(WamWord sora_word, WamWord count_word)
   int offset, char_count, line_count, line_pos;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   Stream_Get_Position(pstm, &offset, &char_count, &line_count, &line_pos);
 
@@ -1130,7 +1262,7 @@ Line_Count_2(WamWord sora_word, WamWord count_word)
   int offset, char_count, line_count, line_pos;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (!pstm->prop.text)
     Pl_Err_Permission(permission_operation_access,
@@ -1156,7 +1288,7 @@ Line_Position_2(WamWord sora_word, WamWord count_word)
   int offset, char_count, line_count, line_pos;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (!pstm->prop.text)
     Pl_Err_Permission(permission_operation_access,
@@ -1182,7 +1314,7 @@ Stream_Line_Column_3(WamWord sora_word, WamWord line_word, WamWord col_word)
   int offset, char_count, line_count, line_pos;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (!pstm->prop.text)
     Pl_Err_Permission(permission_operation_access,
@@ -1211,7 +1343,7 @@ Set_Stream_Line_Column_3(WamWord sora_word, WamWord line_word,
 
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
   if (!pstm->prop.reposition)
     Pl_Err_Permission(permission_operation_reposition,
@@ -1266,8 +1398,7 @@ Open_Input_Term_Stream_2(WamWord sink_term_word, WamWord stm_word)
 	Rd_Codes_Str_Check(sink_term_word, str);
     }
 
-  stm = Add_Str_Stream(FALSE, str);
-  stm_tbl[stm].prop.other = SYS_VAR_OPTION_MASK + 1;
+  stm = Add_Str_Stream(str, SYS_VAR_OPTION_MASK);
 
   Get_Integer(stm, stm_word);
 }
@@ -1288,11 +1419,11 @@ Close_Input_Term_Stream_1(WamWord sora_word)
   int type;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
-  type = pstm->prop.other - 1;
+  type = pstm->prop.other;
 
-  if (type < 0 || type > 2)
+  if (type < 1 || type > 3)
     Pl_Err_Domain(domain_term_stream_or_alias, sora_word);
 
   if (pstm->prop.output)
@@ -1320,8 +1451,7 @@ Open_Output_Term_Stream_1(WamWord stm_word)
 {
   int stm;
 
-  stm = Add_Str_Stream(FALSE, NULL);
-  stm_tbl[stm].prop.other = SYS_VAR_OPTION_MASK + 1;
+  stm = Add_Str_Stream(NULL, SYS_VAR_OPTION_MASK);
 
   Get_Integer(stm, stm_word);
 }
@@ -1342,11 +1472,11 @@ Close_Output_Term_Stream_2(WamWord sora_word, WamWord sink_term_word)
   char *str;
 
   stm = Get_Stream_Or_Alias(sora_word, STREAM_CHECK_EXIST);
-  pstm = stm_tbl + stm;
+  pstm = stm_tbl[stm];
 
-  type = pstm->prop.other - 1;
+  type = pstm->prop.other;
 
-  if (type < 0 || type > 2)
+  if (type < 1 || type > 3)
     Pl_Err_Domain(domain_term_stream_or_alias, sora_word);
 
   if (pstm->prop.input)
