@@ -6,7 +6,7 @@
  * Descr.: term support                                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: term_supp.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: term_supp.c,v 1.7 2002/03/19 18:24:36 diaz Exp $ */
 
 #include <string.h>
 
@@ -180,7 +180,7 @@ terminal_rec:
 
   DEREF(start_word, word, tag_mask);
 
-  switch (Tag_Mask_Of(word))
+  switch (Tag_Of(word))
     {
     case REF:
       (*fct) (UnTag_REF(word), word);
@@ -188,7 +188,7 @@ terminal_rec:
 
 #ifndef NO_USE_FD_SOLVER
     case FDV:
-      if (!generic_var)
+      if (generic_var)
 	(*fct) (UnTag_FDV(word), word);
       break;
 #endif
