@@ -6,7 +6,7 @@
  * Descr.: architecture dependent features - Header file                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,12 +22,12 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: arch_dep.h,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: arch_dep.h,v 1.7 2002/03/19 18:24:36 diaz Exp $ */
 
 #ifndef _ARCH_DEP_H
 #define _ARCH_DEP_H
 
-#ifdef M_ix86_win32
+#if defined(_WIN32) && !defined(__CYGWIN__)
 
 #define MAXPATHLEN                 1024
 #define SIGQUIT                    SIGTERM
@@ -41,8 +41,8 @@
 #define popen                      _popen
 #define pclose                     _pclose
 #define getpid                     _getpid
-#define strcasecmp                 _stricmp
-#define strncasecmp                _strnicmp
+#define strcasecmp                 stricmp
+#define strncasecmp                strnicmp
 #define unlink                     _unlink
 #define tzset                      _tzset
 #define access                     _access
@@ -70,9 +70,9 @@
 #endif
 
 #if defined(M_ix86_cygwin) || defined(M_ix86_sco)
-#define Set_Line_Buf(s)            setvbuf(s,NULL,_IOLBF,0)
+#define Set_Line_Buf(s)            setvbuf(s, NULL, _IOLBF, 0)
 #elif defined(M_ix86_win32)
-#define Set_Line_Buf(s)            setbuf(s,NULL)
+#define Set_Line_Buf(s)            setbuf(s, NULL)
 #else
 #define Set_Line_Buf(s)            setlinebuf(s)
 #endif
@@ -101,6 +101,9 @@
 #define __unix__
 #endif
 
+#ifndef HAVE_FGETC
+#define fgetc getc
+#endif
 
 				/* Fast call macros */
 #if defined(M_ix86)
