@@ -6,7 +6,7 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine.c,v 1.5 2001/04/27 11:28:17 spa Exp $ */
+/* $Id: engine.c,v 1.8 2002/03/19 18:24:36 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,6 +39,10 @@
 
 #include "engine_pl.h"
 
+#ifndef NO_USE_LINEDIT
+#include "../Linedit/linedit.h"
+#endif
+
 
 /*---------------------------------*
  * Constants                       *
@@ -57,6 +61,8 @@
  * Global Variables                *
  *---------------------------------*/
 
+void (*init_stream_supp)();	/* overwritten by foreign if present */
+
 #if !defined(NO_USE_REGS) && NB_OF_USED_MACHINE_REGS > 0
 static WamWord init_buff_regs[NB_OF_USED_MACHINE_REGS];
 #endif
@@ -99,8 +105,8 @@ Start_Prolog(int argc, char *argv[])
   int i, x;
   char *p;
 
-  setlocale (LC_ALL, "");
-  setlocale (LC_NUMERIC, "C");	/* make sure floats come out right... */
+  setlocale(LC_ALL, "");
+  setlocale(LC_NUMERIC, "C");	/* make sure floats come out right... */
 
   Init_Machine();
 
@@ -149,6 +155,16 @@ Start_Prolog(int argc, char *argv[])
   Init_Pred();
   Init_Oper();
 
+#ifndef NO_USE_LINEDIT
+  if (le_initialize != NULL)
+    use_gui = (*le_initialize)();
+  else
+#endif
+    use_gui = 0;
+
+  if (init_stream_supp)
+    (*init_stream_supp)();
+
   Reset_Prolog();
   Fd_Init_Solver();
 
@@ -169,7 +185,7 @@ Stop_Prolog(void)
 {
 #ifdef DEREF_STATS
   double d = (double) chain_len / (double) nb_deref;
-  printf("Deref: nb: %ld  avg len = %g\n", nb_deref, d);
+  fprintf(stderr, "Deref: nb: %ld  avg len = %g\n", nb_deref, d);
 #endif
   Restore_Machine_Regs(init_buff_regs);
 }
