@@ -6,7 +6,7 @@
  * Descr.: general engine - header file                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine.h,v 1.4 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: engine.h,v 1.6 2002/03/19 18:24:36 diaz Exp $ */
 
 #ifdef NO_STACK_TEST
 #   undef  M_Check_Stacks()
@@ -37,10 +37,10 @@
  *---------------------------------*/
 
 
-#define cpp_recurs(p,n)            p##_##n
-#define Prolog_Predicate(p,n)      cpp_recurs(p,n)
+#define cpp_recurs(p, n)           p##_##n
+#define Prolog_Predicate(p, n)     cpp_recurs(p, n)
 
-#define Prolog_Prototype(p,n)      void Prolog_Predicate(p,n)()
+#define Prolog_Prototype(p, n)     void Prolog_Predicate(p, n)()
 
 
 
@@ -65,6 +65,8 @@ char glob_buff[10240];
 long *base_fl;			/* overwritten by foreign if present */
 double *base_fd;		/* overwritten by foreign if present */
 
+int use_gui;
+
 #else
 
 extern int os_argc;
@@ -75,6 +77,8 @@ extern char glob_buff[];
 extern long *base_fl;
 extern double *base_fd;
 
+extern int use_gui;
+
 #endif
 
 
@@ -112,4 +116,4 @@ void Execute_A_Continuation(CodePtr codep);
 
 
 
-#define   Goto_Predicate(p,n)   ((*Prolog_Predicate(p,n))())
+#define   Goto_Predicate(p, n)  ((*Prolog_Predicate(p, n))())
