@@ -6,7 +6,7 @@
  * Descr.: machine dependent features                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.c,v 1.6 2001/02/08 10:24:50 diaz Exp $ */
+/* $Id: machine.c,v 1.13 2002/05/07 17:45:47 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,36 +35,37 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "gp_config.h"
+#include "gp_config.h"		/* ensure __unix__ defined if not Win32 */
 
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
+#if defined(_WIN32) || defined(__CYGWIN__)
+#include <windows.h>
 #endif
 
-#ifdef M_ix86_win32
-#include <windows.h>
-#include <process.h>
-#include <direct.h>
-#else
+#if defined(__unix__) || defined(__CYGWIN__)
 #include <pwd.h>
 #include <unistd.h>
 #include <sys/param.h>
-#endif
-
-#ifdef M_ix86_cygwin
+#include <sys/time.h>
+#include <sys/resource.h>
+#else /* _WIN32 */
 #include <process.h>
-#include <sys/cygwin.h>
-#include <sys/times.h>
+#include <direct.h>
 #endif
 
 #include "engine_pl.h"		/* before netdb.h which declares a function */
 			 /* gcc cannot define a global reg var after a fct */
 
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
 #ifndef NO_USE_SOCKETS
+#  if defined(__unix__) || defined(__CYGWIN__)
 #include <netdb.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
+#  endif
 #define INET_MANAGEMENT
 #endif
 
@@ -114,12 +115,6 @@
  * Global Variables                *
  *---------------------------------*/
 
-#ifdef M_ix86_cygwin
-static long tps = 0;
-static clock_t rt0;
-static struct tms time_buf0;
-#endif
-
 static long start_user_time = 0;
 static long start_system_time = 0;
 static long start_real_time = 0;
@@ -131,9 +126,11 @@ static char cur_work_dir[MAXPATHLEN];
 
 #ifdef M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
 
-static WamWord *check_adr[NB_OF_STACKS + 1];
+static WamWord *check_adr[NB_OF_STACKS];
 
-#else
+#endif
+
+#ifdef M_USE_MMAP
 
 static int page_size;
 
@@ -168,9 +165,9 @@ static char *Host_Name_From_Alias(struct hostent *host_entry);
 
 
 
-#define Round_Up(x,y)           (((x)+(y)-1)/(y)*(y))
+#define Round_Up(x, y)          (((x) + (y) - 1) / (y) * (y))
 
-#define Round_Down(x,y)         ((x)/(y)*(y))
+#define Round_Down(x, y)        ((x) / (y) * (y))
 
 
 
@@ -182,11 +179,6 @@ static char *Host_Name_From_Alias(struct hostent *host_entry);
 void
 Init_Machine(void)
 {
-#ifdef M_ix86_cygwin
-  tps = sysconf(_SC_CLK_TCK);
-  rt0 = times(&time_buf0);
-#endif
-
   tzset();
 
   start_user_time = M_User_Time();
@@ -222,6 +214,7 @@ M_Allocate_Stacks(void)
     len += stk_tbl[i].size;
 
   addr = (WamWord *) Calloc(len, sizeof(WamWord));
+
   if (addr == NULL)
     Fatal_Error(ERR_STACKS_ALLOCATION);
 
@@ -285,25 +278,26 @@ M_Allocate_Stacks(void)
 
 #if !defined(M_ix86_win32) && defined(M_MMAP_HIGH_ADR_ALT)
   i = 0;
-try_mmap:
+ try_mmap:
 #endif
 #ifdef DEBUG
   DBGPRINTF("trying at high addr:%lx\n", (long) addr);
 #endif
   addr = (WamWord *) Round_Down((long) addr, getpagesize());
   addr -= len;
-
 #ifdef M_ix86_win32
   addr = (WamWord *) VirtualAlloc(addr, len,
 				  MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
 #else
   addr = (WamWord *) mmap((caddr_t) addr, len, PROT_READ | PROT_WRITE,
+			  MAP_PRIVATE
+#ifdef MMAP_NEEDS_FIXED
+			  | MAP_FIXED
+#endif
 #ifdef MAP_ANON
-			  MAP_PRIVATE | MAP_ANON, -1,
-#elif defined(M_sparc_sunos) || defined(M_sparc_solaris)
-			  MAP_PRIVATE | MAP_FIXED, fd,
+			  | MAP_ANON, -1,
 #else
-			  MAP_PRIVATE, fd,
+			  , fd,
 #endif
 			  0);
 #endif
@@ -314,11 +308,14 @@ try_mmap:
 #endif
 
 #ifdef M_ix86_win32
-  if (addr == NULL)
+  if (addr == NULL
 #else
   if ((long) addr == -1
-      || ((long) addr & (((1L << TAG_SIZE) - 1) << VALUE_SIZE)) != 0)
 #endif
+#if TAG_SIZE_HIGH > 0
+      || (((unsigned long) (addr) + len) >> (WORD_SIZE - TAG_SIZE_HIGH)) != 0
+#endif
+      )
     {
 #if !defined(M_ix86_win32) && defined(M_MMAP_HIGH_ADR_ALT)
       if (i == 0)
@@ -393,11 +390,15 @@ Fill_Magic_Adr_Table(void)
 
   for (i = 0; i < NB_OF_STACKS; i++)
     {
-      check_adr[i] = stk_tbl[i].stack + stk_tbl[i].size - M_SECURITY_MARGIN;
-      *check_adr[i] = M_MAGIC;
+      if (stk_tbl[i].size == 0)
+	check_adr[i] = (WamWord *) NULL;
+      else
+	{
+	  check_adr[i] = stk_tbl[i].stack + stk_tbl[i].size -
+	    M_SECURITY_MARGIN;
+	  *check_adr[i] = M_MAGIC;
+	}
     }
-
-  check_adr[NB_OF_STACKS] = NULL;
 }
 
 #endif
@@ -580,11 +581,11 @@ SIGSEGV_Handler(int sig)
 void
 M_Check_Magic_Words(void)
 {
-  WamWord **p = (WamWord **) check_adr;
+  int i;
 
-  for (p = (WamWord **) check_adr; *p; p++)
-    if (**p != M_MAGIC)
-      Stack_Overflow(p - (WamWord **) check_adr);
+  for (i = 0; i < NB_OF_STACKS; i++)
+    if (check_adr[i] && *check_adr[i] != M_MAGIC)
+      Stack_Overflow(i);
 }
 
 #endif
@@ -648,24 +649,23 @@ M_Sys_Err_String(int err_no)
 
 
 
-#if defined(__unix__) || defined(__CYGWIN__)
-
-#include <sys/time.h>
-#include <sys/resource.h>
-
+#ifdef __CIGWIN__
+#define ULL unsigned long long
+#else
+#define ULL unsigned __int64
 #endif
 
-/*-------------------------------------------------------------------------*
- * M_USER_TIME                                                             *
- *                                                                         *
- * returns the user time used since the start of the process (in ms).      *
- *-------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------*/
+/* M_USER_TIME                                                             */
+/*                                                                         */
+/* returns the user time used since the start of the process (in ms).      */
+/*-------------------------------------------------------------------------*/
 long
 M_User_Time(void)
 {
   long user_time;
 
-#if defined(__unix__)
+#if defined(__unix__) && !defined(__CYGWIN__)
   struct rusage rsr_usage;
 
   getrusage(RUSAGE_SELF, &rsr_usage);
@@ -673,16 +673,16 @@ M_User_Time(void)
   user_time = (rsr_usage.ru_utime.tv_sec * 1000) +
     (rsr_usage.ru_utime.tv_usec / 1000);
 
-#elif defined(M_ix86_cygwin)	/* does not work well, returns real_time */
+#elif defined(_WIN32) || defined(__CYGWIN__)
+  FILETIME creat_t, exit_t, kernel_t, user_t;
 
-  struct tms time_buf1;
-
-  times(&time_buf1);
-  user_time = (time_buf1.tms_utime - time_buf0.tms_utime) * 1000 / tps;
-
-#elif defined(M_ix86_win32)
-
-  user_time = (long) ((double) clock() * 1000 / CLOCKS_PER_SEC);
+  /* Success on Windows NT */
+  if (GetProcessTimes(GetCurrentProcess(),
+		      &creat_t, &exit_t, &kernel_t, &user_t))
+    user_time = (long) (((ULL) user_t.dwHighDateTime << 32) +
+			(ULL) user_t.dwLowDateTime) / 10000;
+  else				/* not implemented on Windows 95/98 */
+    user_time = (long) ((double) clock() * 1000 / CLOCKS_PER_SEC);
 
 #else
 
@@ -707,7 +707,7 @@ M_System_Time(void)
 {
   long system_time;
 
-#if defined(__unix__)
+#if defined(__unix__) && !defined(__CYGWIN__)
   struct rusage rsr_usage;
 
   getrusage(RUSAGE_SELF, &rsr_usage);
@@ -715,16 +715,16 @@ M_System_Time(void)
   system_time = (rsr_usage.ru_stime.tv_sec * 1000) +
     (rsr_usage.ru_stime.tv_usec / 1000);
 
-#elif defined(M_ix86_cygwin)	/* does not work well, returns 0 */
+#elif defined(_WIN32) || defined(__CYGWIN__)
+  FILETIME creat_t, exit_t, kernel_t, user_t;
 
-  struct tms time_buf1;
-
-  times(&time_buf1);
-  system_time = (time_buf1.tms_stime - time_buf0.tms_stime) * 1000 / tps;
-
-#elif defined(M_ix86_win32)
-
-  system_time = 0;
+  /* Success on Windows NT */
+  if (GetProcessTimes(GetCurrentProcess(),
+		      &creat_t, &exit_t, &kernel_t, &user_t))
+    system_time = (long) (((ULL) kernel_t.dwHighDateTime << 32) +
+			  (ULL) kernel_t.dwLowDateTime) / 10000;
+  else				/* not implemented on Windows 95/98 */
+    system_time = 0;
 
 #else
 
@@ -749,15 +749,15 @@ M_Real_Time(void)
 {
   long real_time;
 
-#if defined(__unix__) || defined(__CYGWIN__)
+#if defined(__unix__) && !defined(__CYGWIN__)
   struct timeval tv;
 
   gettimeofday(&tv, NULL);
   real_time = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
 
-#elif defined(M_ix86_win32)
+#elif defined(_WIN32) || defined(__CYGWIN__)
 
-  real_time = M_User_Time();
+  real_time = (long) ((double) clock() * 1000 / CLOCKS_PER_SEC);
 
 #else
 
@@ -911,7 +911,8 @@ M_Host_Name_From_Adr(char *host_address)
   struct in_addr iadr;
 
 #if defined(M_sparc_sunos) || defined(M_sparc_solaris) || \
-    defined(M_ix86_cygwin) || defined(M_ix86_solaris)
+    defined(M_ix86_cygwin) || defined(M_ix86_solaris)  || \
+    defined(M_ix86_win32)
   if ((iadr.s_addr = inet_addr(host_address)) == -1)
 #else
   if (inet_aton(host_address, &iadr) == 0)
@@ -1051,7 +1052,7 @@ M_Absolute_Path_Name(char *src)
     }
   *dst = '\0';
 
-#if defined(M_ix86_win32) || defined(M_ix86_cygwin)
+#if defined(_WIN32) || defined(__CYGWIN__)
   for (src = buff[res]; *src; src++)	/* \ becomes / */
     if (*src == '\\')
       *src = '/';
@@ -1093,7 +1094,7 @@ M_Absolute_Path_Name(char *src)
 #endif
     }
 
-#ifdef M_ix86_cygwin
+#ifdef __CYGWIN__
   cygwin_conv_to_full_posix_path(buff[res], buff[1 - res]);
   res = 1 - res;
 #endif
@@ -1120,7 +1121,7 @@ M_Absolute_Path_Name(char *src)
 	continue;
 
     collapse:
-#ifndef M_ix86_cygwin		/* CYGWIN uses //<drive>/<path> for <drive>:\<path> */
+#ifndef __CYGWIN__		/* CYGWIN uses //<drive>/<path> for <drive>:\<path> */
       while (*src == DIR_SEP_C)	/* collapse /////... as / */
 	src++;
 #endif
