@@ -6,7 +6,7 @@
  * Descr.: machine dependent features - Header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.h,v 1.4 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: machine.h,v 1.9 2002/05/10 16:35:25 diaz Exp $ */
 
 #include "bool.h"
 
@@ -97,24 +97,24 @@ void SIGSEGV_Handler(void);
 
 #if defined(M_sparc_sunos)
 
-#    define M_USED_REGS            {"g6","g7",0}
+#    define M_USED_REGS            {"g6", "g7", 0}
 
 #elif defined(M_sparc_solaris)
 
-#    define M_USED_REGS            {"g6","g7",0}
+#    define M_USED_REGS            {"g6", "g7", 0}
 
 #elif defined(M_mips_irix)
 
-#define M_USED_REGS                {"$16","$17","$18","$19","$20",\
-                                    "$21","$22","$23",0}
+#define M_USED_REGS                {"$16", "$17", "$18", "$19", "$20", \
+                                    "$21", "$22", "$23", 0}
 
 #elif defined(M_alpha_linux)
 
-#    define M_USED_REGS            {"$9","$10","$11","$12","$13","$14",0}
+#    define M_USED_REGS            {"$9", "$10", "$11", "$12", "$13", "$14", 0}
 
 #elif defined(M_alpha_osf)
 
-#    define M_USED_REGS            {"$9","$10","$11","$12","$13","$14",0}
+#    define M_USED_REGS            {"$9", "$10", "$11", "$12", "$13", "$14", 0}
 
 #elif defined(M_ix86_linux)   || defined(M_ix86_sco) || \
       defined(M_ix86_solaris) || defined(M_ix86_cygwin)
@@ -127,7 +127,11 @@ void SIGSEGV_Handler(void);
 
 #elif defined(M_powerpc_linux)
 
-#    define M_USED_REGS            {"15","20",0}
+#    define M_USED_REGS            {"15", "20", 0}
+
+#elif defined(M_powerpc_darwin)
+
+#    define M_USED_REGS            {"15", "20", 0}
 
 #else
 
@@ -147,10 +151,10 @@ void SIGSEGV_Handler(void);
  * Stacks Management               *
  *---------------------------------*/
 
-#if defined(M_sparc_sunos) || defined(M_sparc_solaris) || \
-    defined(M_ix86_linux)  || defined(M_powerpc_linux) || \
-    defined(M_ix86_sco)    || defined(M_ix86_solaris)  || \
-    defined(M_mips_irix)   || \
+#if defined(M_sparc_sunos) || defined(M_sparc_solaris)  || \
+    defined(M_ix86_linux)  || defined(M_powerpc_linux)  || \
+    defined(M_ix86_sco)    || defined(M_ix86_solaris)   || \
+    defined(M_mips_irix)   || defined(M_powerpc_darwin) || \
     defined(M_ix86_win32)
 
 #   define M_USE_MMAP
@@ -171,10 +175,13 @@ void SIGSEGV_Handler(void);
 
 #endif
 
+#if defined(M_sunos) || defined(M_solaris)
+#   define MMAP_NEEDS_FIXED
+#endif
 
 
 
-#ifdef M_USE_MALLOC
+#if defined(M_USE_MALLOC) || defined(M_powerpc_darwin)
 
 #define M_USE_MAGIC_NB_TO_DETECT_STACK_NAME
 void M_Check_Magic_Words(void);
