@@ -6,7 +6,7 @@
  * Descr.: main file                                                       * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd2c.pl,v 1.4 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: fd2c.pl,v 1.6 2002/03/19 18:24:36 diaz Exp $ */
 
 fd2c(Args) :-
 	catch(fd2c1(Args), Err, exception(Err)).
@@ -80,7 +80,7 @@ c_file(CFile, _, CFile).
 
 
 
-          /*--- command-line options reading ---*/
+          % Command-line options reading
 
 cmd_line_args(Args, FdFile, CFile) :-
 	g_assign(fdfile, ''),
@@ -159,7 +159,7 @@ cmd_line_arg1(FdFile, LArg, LArg) :-
 
 
 
-          /*--- copying ---*/
+          % Copying
 
 display_copying :-
 	current_prolog_flag(prolog_name, Name),
@@ -177,7 +177,7 @@ display_copying :-
 	
 
 
-          /*--- help ---*/
+          % Help
 
 h('Usage: fd2c [OPTION...] FILE').
 h('').
@@ -191,7 +191,7 @@ h('''user'' can be given as FILE for the standard input/output').
 
 
 
-          /*--- exception recovery ---*/
+          % Exception recovery
 
 exception(error(syntax_error(_), _)) :-
 	!,
@@ -231,7 +231,7 @@ warn(Msg, Args) :-
 
 
 
-          /*--- starting directive ---*/
+          % Starting directive
 
 go :-
 	argument_list(L),
