@@ -6,7 +6,7 @@
  * Descr.: parsing and some code emission                                  * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: parse.pl,v 1.5 2001/09/28 18:26:47 diaz Exp $ */
+/* $Id: parse.pl,v 1.7 2002/03/19 18:24:36 diaz Exp $ */
 
 parse_user_cstr(uc(Name, AFSize, LHVar, Body)) -->
 	head(Name, NbHVar, LHVar),
@@ -726,8 +726,7 @@ sem_error(Msg, Args) -->
 
 
 
-          /* utilities */
-
+          % Utilities
 
 
 add_use_to_dep(LUse, _) :-
