@@ -6,7 +6,7 @@
  * Descr.: test file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,12 +22,16 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: test_linedit.c,v 1.4 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: test_linedit.c,v 1.10 2002/06/12 12:05:28 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <signal.h>
+#include <locale.h>
+#include <time.h>
+
+#include "../EnginePl/gp_config.h"
+
 #if defined(__unix__) || defined(__CYGWIN__)
 #include <unistd.h>
 #else
@@ -35,18 +39,19 @@
 #include <process.h>
 #endif
 
-
+#include "ctrl_c.h"
 #include "linedit.h"
 
+
 #define printf LE_Printf
 
 
-
-
 /*---------------------------------*
  * Constants                       *
  *---------------------------------*/
 
+#define MAX_SIZE 500000
+
 /*---------------------------------*
  * Type Definitions                *
  *---------------------------------*/
@@ -59,53 +64,42 @@
  * Function Prototypes             *
  *---------------------------------*/
 
+
 /*-------------------------------------------------------------------------*
- * INTERRUPT_HANDLER                                                       *
+ * CTRL_C_MANAGER                                                          *
  *                                                                         *
  *-------------------------------------------------------------------------*/
-void
-Interrupt_Handler(int sig)
+long
+Ctrl_C_Manager(int from_callback)
 {
   int c;
   char prefix[100];
   char *str;
   int nb, max_lg, is_last;
 
-#if defined(__unix__) || defined(__CYGWIN__)
-  sigset_t set;
-#endif
+  printf("\nCATCHING CTRL+C prompt length: %d   current pos: %d\n",
+	 LE_Get_Prompt_Length(), LE_Get_Current_Position());
 
-  printf("\nCatching signal:%d\n", sig);
-  printf
-    ("e to exit - c to continue - C for completions - w for current word:");
+  printf("e: exit, c: continue, C: completions, w: current word: ");
   fflush(stdout);
 
-#if defined(__unix__) || defined(__CYGWIN__)
-  sigemptyset(&set);
-  sigaddset(&set, sig);
-  sigprocmask(SIG_UNBLOCK, &set, NULL);
-#else
-  signal(sig, Interrupt_Handler);
-#endif
 
-  c = LE_FGetc_No_Echo(0, 1);
+  c = LE_Get_Key(1, 1);
   printf("\n");
 
-  if (c == 'e')
-    exit(0);
-
-  if (c == 'w')
+  switch (c)
     {
+    case 'e':
+      exit(0);
+
+    case 'w':
       LE_Get_Current_Word(prefix);
       printf("current word=<%s>\n", prefix);
-      return;
-    }
+      break;
 
-  if (c == 'C')
-    {
+    case 'C':
       printf("Enter a prefix:");
-      fgets(prefix, sizeof(prefix), stdin);
-      prefix[strlen(prefix) - 1] = '\0';
+      LE_Gets(prefix);
       if ((str = LE_Compl_Init_Match(prefix, &nb, &max_lg)) == NULL)
 	printf("no matching\n");
       else
@@ -114,12 +108,36 @@ Interrupt_Handler(int sig)
 	  while ((str = LE_Compl_Find_Match(&is_last)) != NULL)
 	    printf("matching: <%s>\n", str);
 	}
+      break;
     }
+
+  return 0;
 }
 
 
 
 
+/*-------------------------------------------------------------------------*
+ * SET_TEST_LOCALE                                                         *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Set_Test_Locale(void)
+{
+  time_t ltime;
+  struct tm *thetime;
+  unsigned char str[100];
+
+  setlocale(LC_ALL, "");
+  time(&ltime);
+  thetime = gmtime(&ltime);
+
+  strftime(str, 100, "%d (%A) %m (%B) %Y", thetime);
+  printf("Date in current locale with strftime: %s\n", str);
+}
+
+
+
 /*-------------------------------------------------------------------------*
  * MAIN                                                                    *
  *                                                                         *
@@ -127,36 +145,61 @@ Interrupt_Handler(int sig)
 int
 main(int argc, char *argv[])
 {
-  char line[256];
+  static char line[MAX_SIZE];
   char *p;
   char sep[100];
+  int ret_val;
+  int tempo = 0;
 
-#if defined(_MSC_VER)
+#if defined(_WIN32) && !defined(__CYGWIN__)
   setbuf(stdout, NULL);
   setbuf(stderr, NULL);
 #endif
 
 #if 1
-  signal(SIGINT, Interrupt_Handler);
+  Install_Ctrl_C_Handler(Ctrl_C_Manager);
 #endif
 
+  Set_Test_Locale();
+
+  if (argc > 1)
+    tempo = atoi(argv[1]);
+
   sep[0] = '\n';
   strcpy(sep + 1, LE_Get_Separators());
 
   printf("enter lines (EOF to finish)\n");
 
-  for (;;)
-    {
 #if 0
-      printf("tempo to allow you to buffer some chars before the read\n");
-      sleep(2);
+  {				/* test space overflow in WIN32 GUI console */
+    int i;
+    char buf[256];
+
+    for (i = 0; i < 280; i++)
+      {
+	sprintf(buf,
+		"line %3d tttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt\n",
+		i);
+	printf(buf);
+      }
+
+  }
 #endif
 
+  for (;;)
+    {
+#ifdef __unix__
+      if (tempo)
+	{
+	  printf("tempo %d secs to allow you to buffer some chars:\n");
+	  sleep(tempo);
+	}
+#endif
 #if 0
       printf("enter a line:");
       if (LE_Gets(line) == NULL)
 #else
-      if (LE_FGets(line, 200, 0, 1, "enter a line:", 1) == NULL)
+      if (LE_FGets(line, MAX_SIZE, "enter a line:", 1) == NULL)
 #endif
 	break;
       printf("Line:(%s) len:%d\n", line, strlen(line));
@@ -169,17 +212,8 @@ main(int argc, char *argv[])
 
   printf("End of testing\n");
 
-  return 12;
+  ret_val = 12;
+  if (le_hook_exit_process)
+    (*le_hook_exit_process) (ret_val);
+  return ret_val;
 }
-
-
-/* To Compile with W32GUICons */
-char **
-M_Cmd_Line_To_Argv(char *cmd,int *argc)
-{
-  *argc = 0;
-  return NULL;
-}
-
-
-
