@@ -6,7 +6,7 @@
  * Descr.: test file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,9 +22,11 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: test_noecho.c,v 1.3 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: test_noecho.c,v 1.5 2002/03/19 18:24:37 diaz Exp $ */
 
 #include <stdio.h>
+#include <locale.h>
+#include <ctype.h>
 #include <stdarg.h>
 
 #include "linedit.h"
@@ -58,12 +60,33 @@
  *                                                                         *
  *-------------------------------------------------------------------------*/
 int
-main()
+main(int argc, char *argv[])
 {
   int c;
 
+  setlocale(LC_ALL, "");
+
   while ((c = LE_FGetc_No_Echo(0, 1)) != EOF)
-    printf("Read Char (%d)=%c\n", c, c);
+    {
+      printf("Read Char (%d)", c);
+      if (isprint(c))
+	printf(" = %c", c);
+      if (isalpha(c))
+	printf(" isalpha");
+      if (isascii(c))
+	printf(" isascii");
+      if (isupper(c))
+	printf(" isupper");
+      if (islower(c))
+	printf(" islower");
+      if (isspace(c))
+	printf(" isspace");
+      if (isprint(c))
+	printf(" isprint");
+      if (isgraph(c))
+	printf(" isgraph");
+      printf("\n");
+    }
 
   return 0;
 }
