@@ -1,15 +1,15 @@
-/*-------------------------------------------------------------------------*/
-/* Prolog To Wam Compiler                INRIA Rocquencourt - CLoE Project */
-/* C Run-time                                           Daniel Diaz - 1994 */
-/*                                                                         */
-/* Machine Dependent Features - Header file                                */
-/*                                                                         */
-/* machine.h                                                               */
-/*-------------------------------------------------------------------------*/
+/*-------------------------------------------------------------------------* 
+ * Prolog To Wam Compiler                INRIA Rocquencourt - CLoE Project * 
+ * C Run-time                                           Daniel Diaz - 1994 * 
+ *                                                                         * 
+ * Machine Dependent Features - Header file                                * 
+ *                                                                         * 
+ * machine.h                                                               * 
+ *-------------------------------------------------------------------------*/
 
-/*---------------------------------*/
-/* Asm Labels, Symbols and Gotos   */
-/*---------------------------------*/
+/*---------------------------------* 
+ * Asm Labels, Symbols and Gotos   * 
+ *---------------------------------*/
 
 #if defined(M_sony_news) || defined(M_ultrix_dec) || defined(M_alpha_osf) ||\
     defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd)
@@ -79,54 +79,26 @@
 
 
 
-/*---------------------------------*/
-/* WAM                             */
-/*---------------------------------*/
-
-typedef long WamWord;
-typedef WamWord *WamWordP;
-typedef void(*WamCont)();
-typedef void  (*CodePtr)();  
-
-#if 1
-
-#if defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd)
-
-register WamWord                *reg_bank asm ("ebx");
-
-#elif defined(M_powerpc_linux)
-
-register WamWord                *reg_bank asm ("r31");
-
-#else
-
-WamWord *reg_bank;
-
-#endif
-
-#else
-
-WamWord *reg_bank;
-
-#endif
-
+/*---------------------------------* 
+ * WAM                             * 
+ *---------------------------------*/
 
 #if 0
 
+#if defined(M_ix86_linux) || defined(M_ix86_sco) || defined(M_ix86_bsd)
+
+register WamWord *reg_bank asm("ebx");
+
+#elif defined(M_powerpc_linux)
+
+register WamWord *reg_bank asm("r31");
+
 #else
 
-#define B                       ((WamWordP) (reg_bank[NB_OF_X_REGS+1]))
-#define CP                      ((WamCont)   (reg_bank[NB_OF_X_REGS+3]))
-#define E                       ((WamWordP) (reg_bank[NB_OF_X_REGS+4]))
+WamWord *reg_bank;
 
-#define ALTB(b)                    ((CodePtr)   (b[-1]))
+#endif
 
 #endif
 
 
-#define X(x)                       (reg_bank[x])
-#define Y(e,y)                     ((WamWord)   (e[-4-(y)]))
-
-
-
-
