@@ -6,7 +6,7 @@
  * Descr.: translation file for Win32 on intel x86 (for nasm)              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ix86_win32.c,v 1.6 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: ix86_win32.c,v 1.10 2002/06/12 12:05:29 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -255,6 +255,32 @@ Pl_Jump(char *label)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PREP_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Prep_CP(void)
+{
+  Inst_Printf("mov", "dword %s,.Lcont%d", asm_reg_cp, w_label);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
+  Label_Printf(".Lcont%d:", w_label++);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CALL                                                                 *
  *                                                                         *
@@ -264,9 +290,9 @@ Pl_Call(char *label)
 {
   BT_String_Add(&bt_symbol, strdup(label));
 
-  Inst_Printf("mov", "dword %s,.Lcont%d", asm_reg_cp, w_label);
+  Prep_CP();
   Pl_Jump(label);
-  Label_Printf(".Lcont%d:", w_label++);
+  Here_CP();
 }
 
 
@@ -337,7 +363,7 @@ Move_From_Reg_X(int index)
 void
 Move_From_Reg_Y(int index)
 {
-  Inst_Printf("mov", "eax,[-%d+%s]", (index + 4) * 4, asm_reg_e);
+  Inst_Printf("mov", "eax,[%d+%s]", Y_OFFSET(index), asm_reg_e);
 }
 
 
@@ -363,7 +389,7 @@ Move_To_Reg_X(int index)
 void
 Move_To_Reg_Y(int index)
 {
-  Inst_Printf("mov", "[-%d+%s],eax", (index + 4) * 4, asm_reg_e);
+  Inst_Printf("mov", "[%d+%s],eax", Y_OFFSET(index), asm_reg_e);
 }
 
 
@@ -492,12 +518,12 @@ Call_C_Arg_Reg_Y(int offset, int adr_of, int index)
 {
   if (adr_of)
     {
-      Inst_Printf("lea", "eax,[-%d+%s]", (index + 4) * 4, asm_reg_e);
+      Inst_Printf("lea", "eax,[%d+%s]", Y_OFFSET(index), asm_reg_e);
       Inst_Printf("mov", "[esp+%d],eax", offset * 4);
     }
   else
     {
-      Inst_Printf("mov", "eax,[-%d+%s]", (index + 4) * 4, asm_reg_e);
+      Inst_Printf("mov", "eax,[%d+%s]", Y_OFFSET(index), asm_reg_e);
       Inst_Printf("mov", "[esp+%d],eax", offset * 4);
     }
 
@@ -651,7 +677,7 @@ Move_Ret_To_Reg_X(int index)
 void
 Move_Ret_To_Reg_Y(int index)
 {				/* same as Move_To_Reg_Y */
-  Inst_Printf("mov", "[-%d+%s],eax", (index + 4) * 4, asm_reg_e);
+  Inst_Printf("mov", "[%d+%s],eax", Y_OFFSET(index), asm_reg_e);
 }
 
 
