@@ -6,7 +6,7 @@
  * Descr.: compiler main (shell) program                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_comp.c,v 1.7 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: top_comp.c,v 1.11 2002/05/10 16:35:26 diaz Exp $ */
 
 #include "../EnginePl/gp_config.h"
 
@@ -40,25 +40,18 @@
 #include <process.h>
 #include <fcntl.h>
 #include <io.h>
-#define StrStr(s1,s2) Str_Case_Str(s1,s2)
-char *Str_Case_Str(char *s1, char *s2);
 #else
 #include <dirent.h>
 #include <unistd.h>
 #include <sys/param.h>
 #include <sys/wait.h>
-#define StrStr(s1,s2) strstr(s1,s2)
 #endif
 
-#ifdef M_ix86_cygwin
-#include <process.h>
-#endif
-
-#include "../EnginePl/arch_dep.h"
 #include "../EnginePl/wam_regs.h"
 
 #include "decode_hexa.c"
 #include "copying.c"
+#include "prolog_path.c"
 
 #include "../EnginePl/machine1.c"
 
@@ -73,12 +66,8 @@ char *Str_Case_Str(char *s1, char *s2);
  * Constants                       *
  *---------------------------------*/
 
-#define MAX_SUB_DIRS               128
-
-#define MAX_FILES                  1024
-
 #define CMD_LINE_MAX_OPT           4096
-#define CMD_LINE_LENGTH            (MAXPATHLEN+CMD_LINE_MAX_OPT+1)
+#define CMD_LINE_LENGTH            (MAXPATHLEN + CMD_LINE_MAX_OPT + 1)
 
 #define TEMP_FILE_PREFIX           GPLC
 
@@ -111,6 +100,8 @@ char *Str_Case_Str(char *s1, char *s2);
 #define FILE_C                     6
 #define FILE_LINK                  7
 
+#define LINK_OPTION                8
+
 
 
 
@@ -121,7 +112,7 @@ char *Str_Case_Str(char *s1, char *s2);
 #define MA_SUFFIX                  ".ma"
 #define FD_SUFFIX                  ".fd"
 #define C_SUFFIX                   ".c"
-#define C_SUFFIX_ALTERNATE         "|.C|.CC|.cc|.cxx|.c++|.cpp|"
+#define C_SUFFIX_ALTERNATE         "|.C|.cc|.CC|.cxx|.CXX|.c++|.C++|.cpp|.CPP|"
 
 #define CC_COMPILE_OPT             "-c "
 #define CC_INCLUDE_OPT             "-I"
@@ -136,6 +127,7 @@ typedef struct
 {
   char *name;
   char *suffix;
+  char *file_part;
   int type;
   char *work_name1;
   char *work_name2;
@@ -161,10 +153,13 @@ CmdInf;
 char *start_path;
 
 int devel_mode = 0;
-char *devel_dir[MAX_SUB_DIRS];
+char *devel_dir[] = {
+  "EnginePl", "BipsPl", "EngineFD", "BipsFD", "Linedit", "W32GUICons",
+  NULL };
 
-FileInf file[MAX_FILES];
-int nb_file = 0;
+
+FileInf *file_lopt;
+int nb_file_lopt = 0;
 
 int stop_after = FILE_LINK;
 int verbose = 0;
@@ -189,7 +184,7 @@ int strip = 0;
 
 int no_decode_hex = 0;
 
-char warn[1024] = "";
+char warn_str[1024] = "";
 
 char *temp_dir = NULL;
 int no_del_temp_files = 0;
@@ -209,8 +204,7 @@ char *cc_fd2c_flags = CFLAGS " ";
 
 char *suffixes[] =
   { PL_SUFFIX, WAM_SUFFIX, MA_SUFFIX, ASM_SUFFIX, OBJ_SUFFIX,
-  FD_SUFFIX, C_SUFFIX, NULL
-};
+  FD_SUFFIX, C_SUFFIX, NULL };
 
 
 
@@ -221,14 +215,12 @@ char *suffixes[] =
 
 char *Search_Path(char *file);
 
-void Init_Develop_Dir(void);
-
-void Init_Develop_Dir_Rec(char *path, char ***p);
-
 void Determine_Pathnames(void);
 
 void Compile_Files(void);
 
+void Create_Output_File_Name(FileInf *f, char *buff);
+
 void New_Work_File(FileInf *f, int stage, int stop_after);
 
 void Free_Work_File2(FileInf *f);
@@ -253,27 +245,30 @@ void Display_Help(void);
 
 
 
-#define Record_Link_Warn_Option(i) sprintf(warn+strlen(warn),"%s ",argv[i])
+#define Record_Link_Warn_Option(i) \
+  sprintf(warn_str + strlen(warn_str), "%s ", argv[i])
 
 
 
-#define Before_Cmd(cmd)                                                     \
-     if (verbose)                                                           \
-         fprintf(stderr,"%s\n",cmd)
 
-#define After_Cmd(error)                                                    \
-     if (error)                                                             \
-         Fatal_Error("compilation failed")
+#define Before_Cmd(cmd)				\
+  if (verbose)					\
+    fprintf(stderr, "%s\n", cmd)
+
+
+#define After_Cmd(error)			\
+  if (error)					\
+    Fatal_Error("compilation failed")
 
 
 
 char *last_opt;
 
-#define Check_Arg(i,str)           (last_opt=str,strncmp(argv[i],str,strlen(argv[i]))==0)
+#define Check_Arg(i, str)   (last_opt = str, strncmp(argv[i], str, strlen(argv[i])) == 0)
 
-#define Add_Last_Option(opt)       sprintf(opt+strlen(opt),"%s ",last_opt)
+#define Add_Last_Option(opt)       sprintf(opt + strlen(opt), "%s ", last_opt)
 
-#define Add_Option(i,opt)          sprintf(opt+strlen(opt),"%s ",argv[i])
+#define Add_Option(i, opt)         sprintf(opt + strlen(opt), "%s ", argv[i])
 
 
 
@@ -285,93 +280,32 @@ char *last_opt;
 int
 main(int argc, char *argv[])
 {
-  static char resolved[MAXPATHLEN + 1];	/* realpath() fills up to MAXPATHLEN (NULL) */
-  static char buff[MAXPATHLEN];
-  char *p;
+  char **pdev;
 
 #ifdef M_ix86_win32
   setbuf(stdout, NULL);
   setbuf(stderr, NULL);
 #endif
 
+  file_lopt = (FileInf *) calloc(argc + 1, sizeof(FileInf));
+  if (file_lopt == NULL)
+    Fatal_Error("memory allocation fault");
+
   Parse_Arguments(argc, argv);
   if (verbose)
     fprintf(stderr, "\n");
 
-  if ((start_path = getenv(ENV_VARIABLE)) != NULL)
-    goto path_found;
-
-  strcpy(buff, argv[0]);
-
-#ifdef DEBUG
-  fprintf(stderr, "value of argv[0]: <%s>\n", buff);
-#endif
-
-  if (strcasecmp(buff + strlen(buff) - strlen(EXE_SUFFIX), EXE_SUFFIX) != 0)
-    strcat(buff, EXE_SUFFIX);
-
-#ifdef DEBUG
-  fprintf(stderr, "value of argv[0] with suffix: <%s>\n", buff);
-#endif
-
-#ifndef M_ix86_win32
-  if (access(buff, X_OK) == 0)
-    start_path = buff;
-  else
-#endif
-  if ((start_path = Search_Path(buff)) == NULL)
-    goto path_not_found;
-
-#if defined(__unix__) || defined(__CYGWIN__)
-  if (realpath(start_path, resolved + 1) == NULL)	/* +1 for eventual @ */
-    goto path_not_found;
-#else
-  strcpy(resolved + 1, start_path);	/* +1 for eventual @ */
-#endif
-
-#ifdef DEBUG
-  fprintf(stderr, "path of the executable: %s\n", start_path);
-#endif
-
-  sprintf(buff, DIR_SEP_S "bin" DIR_SEP_S "%s", GPLC);
-
-  if ((p = StrStr(resolved + 1, buff)) != NULL)
-    {
-      *p = '\0';
-      start_path = resolved + 1;
-      goto path_found;
-    }
-
-  sprintf(buff, DIR_SEP_S "TopComp" DIR_SEP_S "%s", GPLC);
-
-#ifdef DEBUG
-  fprintf(stderr, "path of the executable: %s\n", start_path);
-  fprintf(stderr, "sub-path to search:     %s\n", buff);
-#endif
-  if ((p = StrStr(resolved + 1, buff)) != NULL)
-    {
-      *p = '\0';
-      start_path = resolved;
-      *start_path = '@';	/* to enforce development mode */
-      goto path_found;
-    }
-
-path_not_found:
-  Fatal_Error
-    ("cannot find the path for %s, set the environment variable %s",
-     PROLOG_NAME, ENV_VARIABLE);
-
-path_found:
-  if (*start_path == '@')	/* development mode */
-    {
-      start_path++;
-      devel_mode = 1;
-    }
+  start_path = Get_Prolog_Path(&devel_mode);
+  if (start_path == NULL)
+    Fatal_Error("cannot find the path for %s, set environment variable %s",
+		PROLOG_NAME, ENV_VARIABLE);
 
   strcat(cmd_cc.opt, CFLAGS_MACHINE " " CFLAGS_REGS CC_COMPILE_OPT);
 
   if (devel_mode)
-    Init_Develop_Dir();
+    for (pdev = devel_dir; *pdev; pdev++)
+      sprintf(cmd_cc.opt + strlen(cmd_cc.opt), "%s%s" DIR_SEP_S "%s ",
+	      CC_INCLUDE_OPT, start_path, *pdev);
   else
     sprintf(cmd_cc.opt + strlen(cmd_cc.opt), "%s%s" DIR_SEP_S "include ",
 	    CC_INCLUDE_OPT, start_path);
@@ -390,160 +324,6 @@ path_found:
 
 
 
-/*-------------------------------------------------------------------------*
- * SEARCH_PATH                                                             *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-char *
-Search_Path(char *file)
-{
-#ifndef M_ix86_win32
-
-  char *path = getenv("PATH");
-  char *p;
-  int l;
-  static char buff[MAXPATHLEN];
-
-  if (path == NULL)
-    return NULL;
-
-  p = path;
-  for (;;)
-    {
-      if ((p = strchr(path, ':')) != NULL)
-	{
-	  l = p - path;
-	  strncpy(buff, path, l);
-	}
-      else
-	{
-	  strcpy(buff, path);
-	  l = strlen(buff);
-	}
-
-      buff[l++] = DIR_SEP_C;
-
-      strcpy(buff + l, file);
-
-      if (access(buff, X_OK) == 0)
-	return buff;
-
-      if (p == NULL)
-	break;
-
-      path = p + 1;
-    }
-
-  return NULL;
-
-#else
-
-  static char buff[MAXPATHLEN];
-  char *file_part;
-
-  SearchPath(NULL, file, ".exe", MAXPATHLEN, buff, &file_part);
-
-  return buff;
-
-#endif
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * INIT_DEVELOP_DIR                                                        *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-void
-Init_Develop_Dir(void)
-{
-  char **p;
-
-  p = devel_dir;
-
-  Init_Develop_Dir_Rec(start_path, &p);
-
-  *p = NULL;
-}
-
-
-
-
-/*-------------------------------------------------------------------------*
- * INIT_DEVELOP_DIR                                                        *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-void
-Init_Develop_Dir_Rec(char *path, char ***p)
-{
-#ifdef M_ix86_win32
-  HANDLE h;
-  WIN32_FIND_DATA d;
-#else
-  DIR *dir;
-  struct dirent *cur_entry;
-  struct stat info;
-#endif
-  char *name;
-  char buff[MAXPATHLEN];
-
-
-#if 0
-  fprintf(stderr, "Searching sub-dir in: %s\n", path);
-#endif
-
-#ifdef M_ix86_win32
-  sprintf(buff, "%s\\*.*", path);
-  h = FindFirstFile(buff, &d);
-  if (h == INVALID_HANDLE_VALUE)
-#else
-  dir = opendir(path);
-  if (dir == NULL)
-#endif
-    Fatal_Error("Cannot access to %s", path);
-
-
-#ifdef M_ix86_win32
-  do
-    {
-      name = d.cFileName;
-#else
-  while ((cur_entry = readdir(dir)) != NULL)
-    {
-      name = cur_entry->d_name;
-#endif
-      sprintf(buff, "%s" DIR_SEP_S "%s", path, name);
-      if (*name != '.' && strcmp(name, "CVS") != 0 &&
-#ifdef M_ix86_win32
-	  (d.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0)
-#else
-	  stat(buff, &info) == 0 && S_ISDIR(info.st_mode))
-#endif
-      {
-#ifdef DEBUG
-	fprintf(stderr, "sub-directory for include: %s (full: %s)\n", name,
-		buff);
-#endif
-	**p = strdup(buff);
-	(*p)++;
-	sprintf(cmd_cc.opt + strlen(cmd_cc.opt), "%s%s ",
-		CC_INCLUDE_OPT, buff);
-	Init_Develop_Dir_Rec(buff, p);
-      }
-#ifdef M_ix86_win32
-    }
-  while (FindNextFile(h, &d) != 0);
-  FindClose(h);
-#else
-    }
-  closedir(dir);
-#endif
-}
-
-
-
-
 /*-------------------------------------------------------------------------*
  * COMPILE_FILES                                                           *
  *                                                                         *
@@ -560,8 +340,8 @@ Compile_Files(void)
 
   if (stop_after < FILE_LINK)
     {
-      if (*warn)
-	fprintf(stderr, "link not done - ignored option(s): %s\n", warn);
+      if (*warn_str)
+	fprintf(stderr, "link not done - ignored option(s): %s\n", warn_str);
 
       stage_end = stop_after;
       needs_stack_file = 0;
@@ -577,7 +357,7 @@ Compile_Files(void)
 
   if (needs_stack_file)
     {
-      f = file + nb_file;
+      f = file_lopt + nb_file_lopt;
 
       f->work_name2 = NULL;
       New_Work_File(f, FILE_WAM, 10000);	/* to create work_name2 */
@@ -610,8 +390,11 @@ Compile_Files(void)
     fprintf(stderr, "\n*** Compiling\n");
 
 
-  for (f = file; f->name; f++)
+  for (f = file_lopt; f->name; f++)
     {
+      if (f->type == LINK_OPTION)
+	continue;
+
       if (verbose &&
 	  (f->type == FILE_FD || f->type == FILE_C || f->type <= stage_end))
 	fprintf(stderr, "\n--- file: %s\n", f->name);
@@ -665,7 +448,7 @@ Compile_Files(void)
 
 	    case FILE_MA:
 	      Compile_Cmd(&cmd_ma2asm, f);
-	      if (needs_stack_file && f == file + nb_file &&
+	      if (needs_stack_file && f == file_lopt + nb_file_lopt &&
 		  !no_del_temp_files)
 		{
 		  if (verbose)
@@ -694,14 +477,81 @@ Compile_Files(void)
   Link_Cmd();
 
   /* removing temp files after link */
-  for (f = file; f->name; f++)
-    if (f->work_name1 != f->name)
+  for (f = file_lopt; f->name; f++)
+    if (f->work_name1 != f->name) /* also ok if f->type == LINK_OPTION */
       Delete_Temp_File(f->work_name1);
 }
 
 
 
 
+/*-------------------------------------------------------------------------*
+ * CREATE_OUTPUT_FILE_NAME                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Create_Output_File_Name(FileInf *f, char *buff)
+{
+  char *p;
+  int l;
+  static int counter = 0;
+
+  for(p = file_name_out; *p; p++)
+    {
+      if (*p != '%')
+	*buff++ = *p;
+      else
+	switch(* ++p)
+	  {
+	  case 'd':		/* %d = the directory part */
+	    strcpy(buff, f->name);
+	    l = f->file_part - f->name;
+	    buff += l;
+	    break;
+
+	  case 'f':		/* %f = the whole file name */
+	    strcpy(buff, f->name);
+	    buff += strlen(buff);
+	    break;
+
+	  case 'F':		/* %F = the whole file name (without dir) */
+	    strcpy(buff, f->file_part);
+	    buff += strlen(buff);
+	    break;
+
+	  case 'p':		/* %p = the prefix file name */
+	    strcpy(buff, f->name);
+	    l = f->suffix - f->name;
+	    buff += l;
+	    break;
+
+	  case 'P':		/* %P = the prefix file name (without dir) */
+	    strcpy(buff, f->file_part);
+	    l = f->suffix - f->file_part;
+	    buff += l;
+	    break;
+
+	  case 's':		/* %s = the suffix */
+	    strcpy(buff, f->suffix);
+	    buff += strlen(buff);
+	    break;
+
+	  case 'c':		/* %c = a counter */
+	    sprintf(buff, "%d", ++counter);
+	    buff += strlen(buff);
+	    break;
+
+	  default:
+	    *buff++ = '%';		/* no special % sequence */
+	    *buff++ = *p;
+	  }
+    }
+  *buff = '\0';
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * NEW_WORK_FILE                                                           *
  *                                                                         *
@@ -720,12 +570,12 @@ New_Work_File(FileInf *f, int stage, int stop_after)
     }
   else				/* final stage */
     if (file_name_out)		/* specified output filename */
-    strcpy(buff, file_name_out);
-  else
-    {
-      strcpy(buff, f->name);
-      strcpy(buff + (f->suffix - f->name), suffixes[stage + 1]);
-    }
+      Create_Output_File_Name(f, buff);
+    else
+      {
+	strcpy(buff, f->name);
+	strcpy(buff + (f->suffix - f->name), suffixes[stage + 1]);
+      }
 
   Free_Work_File2(f);
   f->work_name2 = strdup(buff);
@@ -784,12 +634,13 @@ Link_Cmd(void)
 
 
   if (file_name_out == NULL)
-    {
-      f = file;
-      strcpy(file_out, f->name);
-      file_out[f->suffix - f->name] = '\0';
-      file_name_out = file_out;
-    }
+    file_name_out = "%p";	/* will reuse first file name */
+
+  for (f = file_lopt; f->type == LINK_OPTION; f++)
+    ;				/* use first file name by default */
+
+  Create_Output_File_Name(f, file_out);
+  file_name_out = file_out;
 
   sprintf(buff, "%s%s%s%s ", cmd_link.exe_name, cmd_link.opt,
 	  cmd_link.out_opt, file_name_out);
@@ -797,7 +648,8 @@ Link_Cmd(void)
   Find_File(OBJ_FILE_OBJ_BEGIN, OBJ_SUFFIX, buff + strlen(buff));
   strcat(buff, " ");
 
-  for (f = file; f->name; f++)
+				/* f->work_name1 is OK for LINK_OPTION */
+  for (f = file_lopt; f->name; f++)
     sprintf(buff + strlen(buff), "%s ", f->work_name1);
 
   if (!min_pl_bips)
@@ -860,12 +712,7 @@ Link_Cmd(void)
     {
       if (gui_console)
 	{    /* modify Linedit/Makefile.in to follow this list of ld objects */
-	  Find_File(LIB_W32GUICONS, "", buff + strlen(buff));
-	  strcat(buff, " ");
 	  Find_File("w32gc_interf.obj", "", buff + strlen(buff));
-	  strcat(buff, " ");
-	  Find_File("GUICons.res", "", buff + strlen(buff));
-	  strcat(buff, " /link /subsystem:windows ");
 	}
 #ifdef M_ix86_win32
       else
@@ -1009,7 +856,7 @@ void
 Find_File(char *file, char *suff, char *file_path)
 {
   char name[MAXPATHLEN];
-  char **p;
+  char **pdev;
 
   sprintf(name, "%s%s", file, suff);
   if (!devel_mode)
@@ -1020,9 +867,10 @@ Find_File(char *file, char *suff, char *file_path)
 	return;
     }
   else
-    for (p = devel_dir; *p; p++)
+    for (pdev = devel_dir; *pdev; pdev++)
       {
-	sprintf(file_path, "%s" DIR_SEP_S "%s", *p, name);
+	sprintf(file_path, "%s" DIR_SEP_S "%s" DIR_SEP_S "%s", start_path,
+		*pdev, name);
 	if (access(file_path, F_OK) == 0)
 	  return;
       }
@@ -1033,34 +881,6 @@ Find_File(char *file, char *suff, char *file_path)
 
 
 
-#ifdef M_ix86_win32
-
-/*-------------------------------------------------------------------------*
- * STR_CASE_STR                                                            *
- *                                                                         *
- *-------------------------------------------------------------------------*/
-char *
-Str_Case_Str(char *s1, char *s2)
-{
-  int l1 = strlen(s1);
-  int l2 = strlen(s2);
-  char *end = s1 + l1 - l2;
-
-  while (s1 <= end)
-    {
-      if (strncasecmp(s1, s2, l2) == 0)
-	return s1;
-      s1++;
-    }
-
-  return NULL;
-}
-
-#endif
-
-
-
-
 /*-------------------------------------------------------------------------*
  * FATAL_ERROR                                                             *
  *                                                                         *
@@ -1083,8 +903,9 @@ Fatal_Error(char *format, ...)
   if (verbose)
     fprintf(stderr, "deleting temporary files before exit\n");
 
-  for (f = file; f->name; f++)
+  for (f = file_lopt; f->name; f++)
     {
+				/* also ok if f->type == LINK_OPTION */
       if (f->work_name1 && f->work_name1 != f->name &&
 	  (file_name_out == NULL
 	   || strcasecmp(f->work_name1, file_name_out) != 0))
@@ -1111,7 +932,8 @@ Parse_Arguments(int argc, char *argv[])
 {
   int i, file_name_out_i;
   char **p, *q;
-  FileInf *f = file;
+  FileInf *f = file_lopt;
+  int nb_file = 0;
 
 
   for (i = 1; i < argc; i++)
@@ -1273,7 +1095,7 @@ Parse_Arguments(int argc, char *argv[])
 		Fatal_Error("OPTION missing after %s option", last_opt);
 
 	      Add_Option(i, cmd_cc.opt);
-/* if C options specified do not take into account fd2c default C options */
+	      /* if C options specified do not take into account fd2c default C options */
 	      cc_fd2c_flags = "";
 	      continue;
 	    }
@@ -1424,40 +1246,52 @@ Parse_Arguments(int argc, char *argv[])
 	      if (++i >= argc)
 		Fatal_Error("OPTION missing after %s option", last_opt);
 
-	      Add_Option(i, cmd_link.opt);
 	      Record_Link_Warn_Option(i);
+#if 0
+	      Add_Option(i, cmd_link.opt);
+#else
+	      f->name = f->work_name1 = argv[i];
+	      f->type = LINK_OPTION;
+	      nb_file_lopt++;
+	      f++;
+#endif
 	      continue;
 	    }
 
 	  Fatal_Error("unknown option %s - try %s --help", argv[i], GPLC);
 	}
 
-      if (nb_file == MAX_FILES - 1)	/* reserve 1 for stack sizes file */
-	Fatal_Error("too many files (max=%d)", MAX_FILES);
-
-      nb_file++;
 
       f->name = argv[i];
 
       if ((f->suffix = strrchr(argv[i], '.')) == NULL)
 	f->suffix = argv[i] + strlen(argv[i]);
 
+      for(q = f->suffix; q >= f->name; q--)
+	if (*q == '/'
+#ifdef M_ix86_win32
+	    || *q == '\\'
+#endif
+	    )
+	  break;
+      f->file_part = q + 1;
+
       if (strcasecmp(PL_SUFFIX_ALTERNATE, f->suffix) == 0)
 	f->type = FILE_PL;
       else
-	if ((q = StrStr(C_SUFFIX_ALTERNATE, f->suffix)) &&
+	if ((q = strstr(C_SUFFIX_ALTERNATE, f->suffix)) &&
 	    q[-1] == '|' && q[strlen(f->suffix)] == '|')
-	f->type = FILE_C;
-      else
-	{
-	  f->type = FILE_LINK;
-	  for (p = suffixes; *p; p++)
-	    if (strcasecmp(*p, f->suffix) == 0)
-	      {
-		f->type = p - suffixes;
-		break;
-	      }
-	}
+	  f->type = FILE_C;
+	else
+	  {
+	    f->type = FILE_LINK;
+	    for (p = suffixes; *p; p++)
+	      if (strcasecmp(*p, f->suffix) == 0)
+		{
+		  f->type = p - suffixes;
+		  break;
+		}
+	  }
 
       f->work_name1 = f->name;
       f->work_name2 = NULL;
@@ -1468,11 +1302,13 @@ Parse_Arguments(int argc, char *argv[])
 	  exit(1);
 	}
 
+      nb_file_lopt++;
+      nb_file++;
       f++;
     }
 
 
-  if (f == file)
+  if (f == file_lopt)
     {
       if (verbose)
 	exit(0);		/* --verbose with no files same as --version */
@@ -1482,8 +1318,10 @@ Parse_Arguments(int argc, char *argv[])
 
   f->name = NULL;
 
-  if (nb_file > 1 && stop_after < FILE_LINK && file_name_out)
+  if (nb_file > 1 && stop_after < FILE_LINK && file_name_out &&
+      strchr(file_name_out, '%') == NULL)
     {
+      fprintf(stderr, "named output file ignored with multiples output (or use meta-characters, e.g. %%p)\n");
       Record_Link_Warn_Option(file_name_out_i);
       Record_Link_Warn_Option(file_name_out_i + 1);
       file_name_out = NULL;
@@ -1499,12 +1337,12 @@ Parse_Arguments(int argc, char *argv[])
  *-------------------------------------------------------------------------*/
 void
 Display_Help(void)
-#define L(msg)  fprintf(stderr,"%s\n",msg)
+#define L(msg)  fprintf(stderr, "%s\n", msg)
 {
   fprintf(stderr, "Usage: %s [OPTION]... FILE...\n", GPLC);
   L(" ");
   L("General options:");
-  L("  -o FILE, --output FILE      set output file name");
+  L("  -o FILE, --output FILE      set output file name (see below)");
   L("  -W, --wam-for-native        stop after producing WAM file(s)");
   L("  -w, --wam-for-byte-code     stop after producing WAM for byte-code file(s) (force --no-call-c)");
   L("  -M, --mini-assembly         stop after producing mini-assembly file(s)");
@@ -1568,6 +1406,13 @@ Display_Help(void)
   L("  -s, --strip                 strip the executable");
   L("  -L OPTION                   pass OPTION to the linker");
   L("");
+  L("The file name specified after --output can include meta-characters:");
+  L("  %f for the whole input file name, %F same as %f without directory");
+  L("  %p for the whole prefix name, %P same as %p without directory");
+  L("  %s for the suffix (or empty if not specified)");
+  L("  %d for the directory part (or empty if not specified)");
+  L("  %c for a auto-increment counter");
+  L("");
   L("Report bugs to bug-prolog@gnu.org.");
 }
 
