@@ -6,7 +6,7 @@
  * Descr.: string dico management (file included by wam2ma.c and ma2asm.c) *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bt_string.c,v 1.3 2001/01/23 11:38:21 diaz Exp $ */
+/* $Id: bt_string.c,v 1.5 2002/03/19 18:24:38 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -75,8 +75,8 @@ void BT_String_List(BTString *bt_str, void (*fct) ());
 
 
 
-static
-  BTNode *BT_String_Add_Rec(BTString *bt_str, BTNode **pbt_node, char *str);
+static BTNode *BT_String_Add_Rec(BTString *bt_str, BTNode **pbt_node,
+				 char *str);
 
 static void BT_String_List_Rec(BTNode *bt_node, void (*fct) ());
 
