@@ -6,7 +6,7 @@
  * Descr.: code generation - header file                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_protos.h,v 1.3 2001/01/23 11:38:21 diaz Exp $ */
+/* $Id: wam_protos.h,v 1.5 2002/03/19 18:24:38 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -154,8 +154,4 @@ void F_function(ArgVal arg[]);
 
 void F_call_c(ArgVal arg[]);
 
-void F_call_c_test(ArgVal arg[]);
-
-void F_call_c_jump(ArgVal arg[]);
-
 void F_foreign_call_c(ArgVal arg[]);
